/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.trifork.stamdata;

import java.io.IOException;
import java.util.Optional;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.google.inject.Singleton;
import com.google.inject.servlet.ServletModule;

public class WSDLModule extends ServletModule
{
	
	private final static String startPath = "/service-contract/";
	
	@Override
	protected void configureServlets()
	{
		// We must have a binding to actually tell us if the
		// component is behaving as expected.
		
		requireBinding(WSDLFetcher.class);
		
		// all wsdl files should be in a path starting with /service-contract
		serve(startPath+"*").with(WSDLServlet.class);
	}
	
	@Singleton
	public static class WSDLServlet extends HttpServlet
	{
		private final Provider<WSDLFetcher> wsdl;
		
		@Inject
		WSDLServlet(Provider<WSDLFetcher> wsdl)
		{
			this.wsdl = wsdl;
		}
		
		protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
			String resourcePath = Optional.of(request.getPathInfo()).orElse("");
			
	        Optional<byte[]> content = getResource(resourcePath);
	        if (!content.isPresent()) {
	            response.setStatus(404);
	            return;
	        }
			prepareResponse(response, content.get());
		}

        protected Optional<byte[]> getResource(String resourcePath) throws IOException {
        	return wsdl.get().get(resourcePath);
        }
		
        private void prepareResponse(HttpServletResponse response, byte[] content) throws IOException {
            response.setContentType("text/xml; charset=utf-8");
            response.setContentLength(content.length);
            response.getOutputStream().write(content);
            response.setStatus(200);
        }

        private static final long serialVersionUID = 0L;
	}
		
}
