/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.trifork.stamdata.models;

import static com.trifork.stamdata.Preconditions.checkNotNull;

import java.math.BigInteger;
import java.time.LocalDate;

public class AuthorizationVO extends BaseTemporalVO {

    private BigInteger id;
	private String cpr;
	private String firstName;
	private String lastName;
	private String authorizationCode;
	private String educationCode;
    private String uddannelsesland;
    private String tidsbegraenset;
    private String tilsyn;
    private String tilsynsBemaerkninger;
    private String sanktionsType;
    private String dispensation75Aarsreglen;
    private LocalDate selvstaendigVirkeDato;
    private String selvstaendigtVirke;
    private String selvstaendigvirkeUddLand;
    private String speciale1;
    private LocalDate specialeDato1;
    private String SpecialeUddLand1;
    private String Speciale2;
    private LocalDate SpecialeDato2;
    private String SpecialeUddLand2;
    private String Speciale3;
    private LocalDate SpecialeDato3;
    private String SpecialeUddLand3;
    private String AutorisationGyldig;
    private LocalDate AutorisationsDato;

    public AuthorizationVO() {

    }

    public AuthorizationVO(String cpr, String givenName, String surname, String authorizationCode, String educationCode) {

        this.cpr = checkNotNull(cpr);
        this.firstName = checkNotNull(givenName);
        this.lastName = checkNotNull(surname);
        this.authorizationCode = checkNotNull(authorizationCode);
        this.educationCode = checkNotNull(educationCode);
    }

    public AuthorizationVO(String cpr, String firstName, String lastName, String authorizationCode, String educationCode, String uddannelsesland, String tidsbegraenset, String tilsyn, String tilsynsBemaerkninger, String sanktionsType, String dispensation75Aarsreglen, LocalDate selvstaendigVirkeDato, String selvstaendigtVirke, String selvstaendigvirkeUddLand, String speciale1, LocalDate specialeDato1, String specialeUddLand1, String speciale2, LocalDate specialeDato2, String specialeUddLand2, String speciale3, LocalDate specialeDato3, String specialeUddLand3, String autorisationGyldig, LocalDate autorisationsDato) {
        this.cpr = cpr;
        this.firstName = firstName;
        this.lastName = lastName;
        this.authorizationCode = authorizationCode;
        this.educationCode = educationCode;
        this.uddannelsesland = uddannelsesland;
        this.tidsbegraenset = tidsbegraenset;
        this.tilsyn = tilsyn;
        this.tilsynsBemaerkninger = tilsynsBemaerkninger;
        this.sanktionsType = sanktionsType;
        this.dispensation75Aarsreglen = dispensation75Aarsreglen;
        this.selvstaendigVirkeDato = selvstaendigVirkeDato;
        this.selvstaendigtVirke = selvstaendigtVirke;
        this.selvstaendigvirkeUddLand = selvstaendigvirkeUddLand;
        this.speciale1 = speciale1;
        this.specialeDato1 = specialeDato1;
        this.SpecialeUddLand1 = specialeUddLand1;
        this.Speciale2 = speciale2;
        this.SpecialeDato2 = specialeDato2;
        this.SpecialeUddLand2 = specialeUddLand2;
        this.Speciale3 = speciale3;
        this.SpecialeDato3 = specialeDato3;
        this.SpecialeUddLand3 = specialeUddLand3;
        this.AutorisationGyldig = autorisationGyldig;
        this.AutorisationsDato = autorisationsDato;
    }


    public BigInteger getId() {
        return id;
    }

    public void setId(BigInteger id) {
        this.id = id;
    }

    public String getCpr() {
        return cpr;
    }

    public void setCpr(String cpr) {
        this.cpr = cpr;
    }

    public String getFirstName() {
        return firstName;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public String getAuthorizationCode() {
        return authorizationCode;
    }

    public void setAuthorizationCode(String authorizationCode) {
        this.authorizationCode = authorizationCode;
    }

    public String getEducationCode() {
        return educationCode;
    }

    public void setEducationCode(String educationCode) {
        this.educationCode = educationCode;
    }

    public String getUddannelsesland() {
        return uddannelsesland;
    }

    public void setUddannelsesland(String uddannelsesland) {
        this.uddannelsesland = uddannelsesland;
    }

    public String getTidsbegraenset() {
        return tidsbegraenset;
    }

    public void setTidsbegraenset(String tidsbegraenset) {
        this.tidsbegraenset = tidsbegraenset;
    }

    public String getTilsyn() {
        return tilsyn;
    }

    public void setTilsyn(String tilsyn) {
        this.tilsyn = tilsyn;
    }

    public String getTilsynsBemaerkninger() {
        return tilsynsBemaerkninger;
    }

    public void setTilsynsBemaerkninger(String tilsynsBemaerkninger) {
        this.tilsynsBemaerkninger = tilsynsBemaerkninger;
    }

    public String getSanktionsType() {
        return sanktionsType;
    }

    public void setSanktionsType(String sanktionsType) {
        this.sanktionsType = sanktionsType;
    }

    public String getDispensation75Aarsreglen() {
        return dispensation75Aarsreglen;
    }

    public void setDispensation75Aarsreglen(String dispensation75Aarsreglen) {
        this.dispensation75Aarsreglen = dispensation75Aarsreglen;
    }

    public LocalDate getSelvstaendigVirkeDato() {
        return selvstaendigVirkeDato;
    }

    public void setSelvstaendigVirkeDato(LocalDate selvstaendigVirkeDato) {
        this.selvstaendigVirkeDato = selvstaendigVirkeDato;
    }

    public String getSelvstaendigtVirke() {
        return selvstaendigtVirke;
    }

    public void setSelvstaendigtVirke(String selvstaendigtVirke) {
        this.selvstaendigtVirke = selvstaendigtVirke;
    }

    public String getSelvstaendigvirkeUddLand() {
        return selvstaendigvirkeUddLand;
    }

    public void setSelvstaendigvirkeUddLand(String selvstaendigvirkeUddLand) {
        this.selvstaendigvirkeUddLand = selvstaendigvirkeUddLand;
    }

    public String getSpeciale1() {
        return speciale1;
    }

    public void setSpeciale1(String speciale1) {
        this.speciale1 = speciale1;
    }

    public LocalDate getSpecialeDato1() {
        return specialeDato1;
    }

    public void setSpecialeDato1(LocalDate specialeDato1) {
        this.specialeDato1 = specialeDato1;
    }

    public String getSpecialeUddLand1() {
        return SpecialeUddLand1;
    }

    public void setSpecialeUddLand1(String specialeUddLand1) {
        SpecialeUddLand1 = specialeUddLand1;
    }

    public String getSpeciale2() {
        return Speciale2;
    }

    public void setSpeciale2(String speciale2) {
        Speciale2 = speciale2;
    }

    public LocalDate getSpecialeDato2() {
        return SpecialeDato2;
    }

    public void setSpecialeDato2(LocalDate specialeDato2) {
        SpecialeDato2 = specialeDato2;
    }

    public String getSpecialeUddLand2() {
        return SpecialeUddLand2;
    }

    public void setSpecialeUddLand2(String specialeUddLand2) {
        SpecialeUddLand2 = specialeUddLand2;
    }

    public String getSpeciale3() {
        return Speciale3;
    }

    public void setSpeciale3(String speciale3) {
        Speciale3 = speciale3;
    }

    public LocalDate getSpecialeDato3() {
        return SpecialeDato3;
    }

    public void setSpecialeDato3(LocalDate specialeDato3) {
        SpecialeDato3 = specialeDato3;
    }

    public String getSpecialeUddLand3() {
        return SpecialeUddLand3;
    }

    public void setSpecialeUddLand3(String specialeUddLand3) {
        SpecialeUddLand3 = specialeUddLand3;
    }

    public String getAutorisationGyldig() {
        return AutorisationGyldig;
    }

    public void setAutorisationGyldig(String autorisationGyldig) {
        AutorisationGyldig = autorisationGyldig;
    }

    public LocalDate getAutorisationsDato() {
        return AutorisationsDato;
    }

    public void setAutorisationsDato(LocalDate autorisationsDato) {
        AutorisationsDato = autorisationsDato;
    }

    @Override
    public String toString() {
        return "AuthorizationVO{" +
                "id=" + id +
                ", cpr='" + cpr + '\'' +
                ", firstName='" + firstName + '\'' +
                ", lastName='" + lastName + '\'' +
                ", authorizationCode='" + authorizationCode + '\'' +
                ", educationCode='" + educationCode + '\'' +
                ", uddannelsesland='" + uddannelsesland + '\'' +
                ", tidsbegraenset='" + tidsbegraenset + '\'' +
                ", tilsyn='" + tilsyn + '\'' +
                ", tilsynsBemaerkninger='" + tilsynsBemaerkninger + '\'' +
                ", sanktionsType='" + sanktionsType + '\'' +
                ", dispensation75Aarsreglen='" + dispensation75Aarsreglen + '\'' +
                ", selvstaendigVirkeDato='" + selvstaendigVirkeDato + '\'' +
                ", selvstaendigtVirke='" + selvstaendigtVirke + '\'' +
                ", selvstaendigvirkeUddLand='" + selvstaendigvirkeUddLand + '\'' +
                ", speciale1='" + speciale1 + '\'' +
                ", specialeDato1='" + specialeDato1 + '\'' +
                ", SpecialeUddLand1='" + SpecialeUddLand1 + '\'' +
                ", Speciale2='" + Speciale2 + '\'' +
                ", SpecialeDato2='" + SpecialeDato2 + '\'' +
                ", SpecialeUddLand2='" + SpecialeUddLand2 + '\'' +
                ", Speciale3='" + Speciale3 + '\'' +
                ", SpecialeDato3='" + SpecialeDato3 + '\'' +
                ", SpecialeUddLand3='" + SpecialeUddLand3 + '\'' +
                ", AutorisationGyldig='" + AutorisationGyldig + '\'' +
                ", AutorisationsDato='" + AutorisationsDato + '\'' +
                '}';
    }
}
