/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.security;

import javax.inject.Named;
import javax.servlet.http.HttpServletRequest;

import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.servlet.RequestScoped;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sdsd.nsp.slalog.ws.SLALoggingServletFilter;

public class SecurityServiceModule extends AbstractModule {
    public static final String ALLOWED_IDWS_AUDIENCE = "allowed.idws.audience";

    public static final String USE_TEST_FEDERATION_PARAMETER = "useSOSITestFederation";
    public static final String USE_MOCKED_FEDERATION_PARAMETER = "useSOSIMockedTestFederation";

    @Override
    protected void configure() {
        // Nothing to do
    }


    @Provides
    public RequestContextProvider requestContextProvider(@Named(ALLOWED_IDWS_AUDIENCE) String allowedAudience) {
        return new RequestContextProvider(new SecurityContextProvider(), new ServiceActorProvider(allowedAudience));
    }

    @Provides
    @RequestScoped
    public SLALogItem provideCurrentSlaLogItem(HttpServletRequest request) {
        return (SLALogItem) request.getAttribute(SLALoggingServletFilter.CURRENT_LOGITEM_REQUEST_KEY);
    }
}
