/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.testing;

import dk.sds.nsp.security.Security;
import dk.sds.nsp.security.SecurityContext;
import dk.sds.nsp.security.SecurityProvider;

import java.time.ZonedDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;

public class SecurityContextTestHelper {
    public static void setContextOrganisation(String cvr) {
        System.setProperty(SecurityProvider.DEVELOPMENT_PROVIDER_PROPERTY, SDMSecurityContextHelperSystem.class.getName());
        SDMTestSecurityContext context = (SDMTestSecurityContext) Security.getSecurityContext();
        context.setGlobalOrganisation(new SecurityContext.Organisation() {
            @Override
            public OrganisationIdentifierFormat getIdentifierFormat() {
                return OrganisationIdentifierFormat.CVR;
            }

            @Override
            public String getIdentifier() {
                return cvr;
            }

            @Override
            public Optional<String> getName() {
                return Optional.of("Statens Serum Institut");
            }
        });
    }

    public static void resetContextOrganisation() {
        setContextOrganisation("33257872");
    }

    public static void setContextCitizen() {
        System.setProperty(SecurityProvider.DEVELOPMENT_PROVIDER_PROPERTY, SDMSecurityContextHelperCitizen.class.getName());
    }

    public static void setContextUserCitizenUser() {
        System.setProperty(SecurityProvider.DEVELOPMENT_PROVIDER_PROPERTY, SDMSecurityContextHelperCitizen.class.getName());
        setContextUserCitizenUser("0103952595");
    }

    public static void setContextUserCitizenUser(String cpr) {
        SDMTestSecurityContext context = (SDMTestSecurityContext) Security.getSecurityContext();
        context.setGlobalActingUser(new TestCitizenUser(cpr));
    }

    public static void setContextUserHealthcareProfessionalUser() {
        setContextUserHealthcareProfessionalUser("0102732379");
    }

    public static void setContextUserHealthcareProfessionalUser(String cpr) {
        System.setProperty(SecurityProvider.DEVELOPMENT_PROVIDER_PROPERTY, SDMSecurityContextHelperHealthCareProfessional.class.getName());
        SDMTestSecurityContext context = (SDMTestSecurityContext) Security.getSecurityContext();
        context.setGlobalActingUser(new HealthcareProfessionalUser(cpr));
        context.setGlobalMessage(new HealcareProfessionalDevelopmentMessage());
    }

    public static void setContextUserSystemUser() {
        System.setProperty(SecurityProvider.DEVELOPMENT_PROVIDER_PROPERTY, SDMSecurityContextHelperSystem.class.getName());
        SDMTestSecurityContext context = (SDMTestSecurityContext) Security.getSecurityContext();
        context.setGlobalActingUser(null);
        context.setThreadActingUser(null);
        context.setGlobalMessage(new SystemDevelopmentMessage());
    }

    public static void setIdCardExpired() {
        SDMTestSecurityContext context = (SDMTestSecurityContext) Security.getSecurityContext();
        context.setGlobalTicket(new ExpiredTicket());
    }

    public static void setIdCardValid() {
        SDMTestSecurityContext context = (SDMTestSecurityContext) Security.getSecurityContext();
        context.setGlobalTicket(new DevelopmentTicket());
    }

    private static final class HealthcareProfessionalUser implements SecurityContext.User {
        private HealthcareProfessionalUser(String cpr) {
            this.cpr = cpr;
        }
        private String cpr;

        public UserType getUserType() {
            return UserType.HealthcareProfessional;
        }

        public PersonIdentifierFormat getIdentifierFormat() {
            return PersonIdentifierFormat.CPR;
        }

        public String getIdentifier() {
            return cpr;
        }

        public Optional<String> getGivenName() {
            return Optional.of("Karl Hoffmann");
        }

        public Optional<String> getSurname() {
            return Optional.of("Svendsen");
        }

        public Optional<Credentials> getCredentials() {
            return Optional.of(new DevelopmentCredentials());
        }

        public Optional<SecurityContext.PersistentUniqueKey> getPersistentUniqueKey() {
            return Optional.of("CVR:25450442-RID:40252666"::toString);
        }

        private static final class DevelopmentCredentials implements Credentials {
            private DevelopmentCredentials() {
            }

            public Optional<String> getAuthorizationCode() {
                return Optional.of("NS362");
            }

            public Optional<String> getEducationCode() {
                return Optional.of("7170");
            }

            public Optional<String> getNationalRole() {
                return null;
            }

            public Optional<String> getUnverifiedRole() {
                return null;
            }

            public Set<String> getPowerOfAttorneyPrivileges() {
                return Collections.emptySet();
            }
        }
    }

    private static final class TestSystemUser implements SecurityContext.User {
        private TestSystemUser() {
        }

        public UserType getUserType() {
            return null;
        }

        public PersonIdentifierFormat getIdentifierFormat() {
            return null;
        }

        public String getIdentifier() {
            return null;
        }

        public Optional<String> getGivenName() {
            return Optional.empty();
        }

        public Optional<String> getSurname() {
            return Optional.empty();
        }

        public Optional<Credentials> getCredentials() {
            return Optional.empty();
        }

        public Optional<SecurityContext.PersistentUniqueKey> getPersistentUniqueKey() {
            return Optional.empty();
        }
    }

    private static final class TestCitizenUser implements SecurityContext.User {

        public TestCitizenUser(String cpr) {
            this.cpr = cpr;
        }
        private String cpr;

        private static final class DevelopmentCredentials implements Credentials {

            @Override
            public Optional<String> getAuthorizationCode() {
                return Optional.of("AABB123");
            }

            @Override
            public Optional<String> getEducationCode() {
                return Optional.of("7170");
            }

            @Override
            public Optional<String> getNationalRole() {
                return null;
            }

            @Override
            public Optional<String> getUnverifiedRole() {
                return null;
            }

            @Override
            public Set<String> getPowerOfAttorneyPrivileges() {
                Set<String> privs = new HashSet<String>();
                privs.add("test");
                return privs;
            }

        }

        @Override
        public UserType getUserType() {
            return UserType.Citizen;
        }

        @Override
        public PersonIdentifierFormat getIdentifierFormat() {
            return PersonIdentifierFormat.CPR;
        }

        @Override
        public String getIdentifier() { return cpr; }

        @Override
        public Optional<String> getGivenName() {
            return Optional.of("Hans Helmuth");
        }

        @Override
        public Optional<String> getSurname() {
            return Optional.of("Hansen");
        }

        @Override
        public Optional<Credentials> getCredentials() {
            return Optional.of(new TestCitizenUser.DevelopmentCredentials());
        }

        @Override
        public Optional<SecurityContext.PersistentUniqueKey> getPersistentUniqueKey() {
            return Optional.empty();
        }
    }

    private static final class SystemDevelopmentMessage implements SecurityContext.Message {

        @Override
        public Optional<String> getIdentifier() { return Optional.of("1234"); }

        @Override
        public Optional<String> getConversationIdentifier() {
            return Optional.of("5678");
        }

        @Override
        public Optional<String> getAction() {
            return Optional.empty();
        }
    }

    private static final class HealcareProfessionalDevelopmentMessage implements SecurityContext.Message {
        private HealcareProfessionalDevelopmentMessage() {
        }

        public Optional<String> getIdentifier() {
            return Optional.of(UUID.randomUUID().toString());
        }

        public Optional<String> getConversationIdentifier() {
            return Optional.of(UUID.randomUUID().toString());
        }

        public Optional<String> getAction() {
            return Optional.of(UUID.randomUUID().toString());
        }
    }

    private static final class ExpiredTicket implements SecurityContext.Ticket {
        private final ZonedDateTime now;

        private ExpiredTicket() {
            this.now = ZonedDateTime.now();
        }

        public boolean isValid() {
            return false;
        }

        public Federation getFederation() {
            return Federation.Development;
        }

        public Optional<String> getAudience() {
            return Optional.of("Http://nsp");
        }

        public Optional<ZonedDateTime> getCreated() {
            return Optional.of(this.now.minus(2L, ChronoUnit.MINUTES));
        }

        public Optional<ZonedDateTime> getValidFrom() {
            return Optional.of(this.now.minus(2L, ChronoUnit.MINUTES));
        }

        public Optional<ZonedDateTime> getValidTo() {
            return Optional.of(this.now.minus(1L, ChronoUnit.MINUTES));
        }

        public Optional<String> createRequestWithContext(org.w3c.dom.Element requestBody, org.w3c.dom.Element... headers) {
            return Optional.empty();
        }
    }

    private static final class DevelopmentTicket implements SecurityContext.Ticket {
        private final ZonedDateTime now;

        private DevelopmentTicket() {
            this.now = ZonedDateTime.now();
        }

        public boolean isValid() {
            return true;
        }

        public Federation getFederation() {
            return Federation.Development;
        }

        public Optional<String> getAudience() {
            return Optional.of("https://fsk");
        }

        public Optional<ZonedDateTime> getCreated() {
            return Optional.of(this.now.minus(1L, ChronoUnit.MINUTES));
        }

        public Optional<ZonedDateTime> getValidFrom() {
            return Optional.of(this.now.minus(1L, ChronoUnit.MINUTES));
        }

        public Optional<ZonedDateTime> getValidTo() {
            return Optional.of(this.now.plus(1L, ChronoUnit.MINUTES));
        }

        public Optional<String> createRequestWithContext(org.w3c.dom.Element requestBody, org.w3c.dom.Element... headers) {
            return Optional.empty();
        }
    }

}
