/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.testing;

import org.junit.Assert;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import java.util.List;

public class StamdataBatchCopyTestHelper {
    public void assertNodesInElementListAreInKeepList(Element anyAsElement, String dataType, String register, String stamdataNameSpaceUriPrefix, List<String> elementsToKeep) {
        // Get all <dataType:dataType> elements
        String stamdataNamespaceUri = stamdataNameSpaceUriPrefix + register;
        NodeList dataTypeNodes = anyAsElement.getElementsByTagNameNS(stamdataNamespaceUri, dataType);
        int count = 0;

        for (int i = 0; i < dataTypeNodes.getLength(); i++) {
            Element personElement = (Element) dataTypeNodes.item(i);
            NodeList childNodes = personElement.getChildNodes();
            for (int j = childNodes.getLength() - 1; j >= 0; j--) {
                Node childNode = childNodes.item(j);

                // Check if the child node is an Element
                if (childNode.getNodeType() != Node.ELEMENT_NODE) {
                    continue;
                }

                // If the child element's name is not in the list of elements to keep, then the test should fail
                Element childElement = (Element) childNode;
                if (elementsToKeep.contains(childElement.getLocalName())) {
                    count++;
                } else {
                    Assert.fail();
                }
            }
        }
        Assert.assertEquals(elementsToKeep.size(), count);
    }
}
