/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.replication.krs.webservice;

import javax.jws.HandlerChain;
import javax.jws.WebService;
import javax.xml.ws.Holder;

import com.trifork.stamdata.SDMException;
import dk.nsi.stamdata.replication.webservice.*;
import dk.nsi.stamdata.security.AccessControl;
import dk.nsi.stamdata.security.RequestContextProvider;
import dk.nsi.stamdata.security.RequestContextVO;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;

import com.google.inject.Inject;
import com.sun.xml.ws.developer.SchemaValidation;
import com.trifork.stamdata.jaxws.GuiceInstanceResolver.GuiceWebservice;

import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.nsi._2011._10._21.stamdatakrs.ObjectFactory;
import dk.nsi._2011._10._21.stamdatakrs.ReplicationFault;
import dk.nsi._2011._10._21.stamdatakrs.ReplicationRequestType;
import dk.nsi._2011._10._21.stamdatakrs.ReplicationResponseType;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;
import dk.nsi._2011._10._21.stamdatakrs.StamdataReplication;

@WebService(endpointInterface="dk.nsi._2011._10._21.stamdatakrs.StamdataReplication")
@HandlerChain(file="handler-chain.xml")
@GuiceWebservice
@SchemaValidation
public class StamdataReplicationImpl implements StamdataReplication {

    private static final Logger logger = Logger.getLogger(StamdataReplicationImpl.class);

    @Inject
    private RequestContextProvider requestContextProvider;

    @Inject
    private AccessControl accessControl;

    @Inject
    private StamdataReplicator stamdataReplicator;

    @Override
    public ReplicationResponseType replicate(Holder<Security> wsseHeader, Holder<Header> medcomHeader, ReplicationRequestType parameters) throws ReplicationFault
    {
        RequestContextVO requestContext = null;
        try {
            requestContext = requestContextProvider.getRequestContext(null, null);
            accessControl.checkAccess(requestContext, false, null);
        } catch (SDMException e) {
            throw new ReplicationFault(e.getMessage(), null, e);
        }
        ReplicationRequest request = toReplicationRequest(parameters);
        Document feedDocument = null;
        try {
            feedDocument = stamdataReplicator.replicate(wsseHeader, medcomHeader, request, requestContext);
        }
        catch(StamdataReplicationFault srf) {
            throw new ReplicationFault(srf.getMessage(), srf.getFaultInfo(), srf);
        }


        // Construct the output container.
        ReplicationResponseType response = new ObjectFactory().createReplicationResponseType();

        response.setAny(feedDocument.getFirstChild());

        // Log that the client successfully accessed the data.
        // Simply for audit purposes.
        //
        // The client details are included in the MDC.
        //
        logger.info("Records fetched, sending response.");

        return response;
    }

    private ReplicationRequest toReplicationRequest(ReplicationRequestType replicationRequestType) {
        ReplicationRequest request = new ReplicationRequest();

        request.setRegister(replicationRequestType.getRegister());
        request.setDatatype(replicationRequestType.getDatatype());
        request.setVersion(replicationRequestType.getVersion());
        request.setRegisterVersion(replicationRequestType.getRegisterVersion());
        request.setOffset(replicationRequestType.getOffset());
        request.setMaxRecords(replicationRequestType.getMaxRecords());
        request.setRequestType(RequestType.KRS);

        return request;
    }
}
