/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.replication.krs.webservice.monitoring;

import com.google.inject.Inject;
import com.jayway.restassured.RestAssured;
import dk.nsi.stamdata.replication.krs.ComponentController;
import dk.nsi.stamdata.replication.krs.webservice.GuiceTestRunner;
import dk.nsi.stamdata.testing.UndertowSlaLogParameters;
import dk.nsi.stamdata.testing.UndertowJaxWsTestServer;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import javax.inject.Named;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static com.jayway.restassured.RestAssured.expect;
import static org.junit.matchers.JUnitMatchers.containsString;

@RunWith(GuiceTestRunner.class)
public class ComponentMonitorIntegrationTest {
    private UndertowJaxWsTestServer undertowTestServer;

    @Inject
    @Named("run.undertow")
    private boolean runUndertow;

    @Inject
    @Named("skrs.endpoint")
    private String skrsEndpoint;

    @Before
    public void setUp() throws Exception {
        if(runUndertow) {
            undertowTestServer = new UndertowJaxWsTestServer(getPortFromEndpoint(), ComponentController.class, new UndertowSlaLogParameters("Stamdata Replication Module", "stamdata-batch-copy"));
            undertowTestServer.start();
        }
        RestAssured.port = getPortFromEndpoint();
    }

    @After
    public void tearDown() throws Exception {
        if(undertowTestServer != null) {
            undertowTestServer.close();
        }
    }

    @Test
    public void testStatusIsOk()
    {
        expect().statusCode(200).body(containsString("200 OK")).when().get("/status");
    }

    private int getPortFromEndpoint() {
        String pattern = "http://.*:(\\d+).*";

        Matcher m = Pattern.compile(pattern).matcher(skrsEndpoint);
        if(m.find()) {
            return Integer.parseInt(m.group(1));
        }
        else {
            return 80;
        }
    }
}
