/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.replication.webservice;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.sql.Date;
import java.time.LocalDate;

import static org.junit.Assert.fail;

@RunWith(GuiceTestRunner.class)
public class DynamicViewXmlGeneratorTest {
    @Test
    public void testDateToString2015(){
        Date date = Date.valueOf(LocalDate.of(2015, 1, 1));
        String actual = DynamicViewXmlGenerator.DateToString(date);
        Assert.assertEquals(date.toString(), actual);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testDateToStringNull(){
        DynamicViewXmlGenerator.DateToString(null);
        fail("Expected IllegalArgumentException");
    }

    @Test
    public void testDateToString1850(){
        Date date = Date.valueOf(LocalDate.of(1850, 1, 1));
        String actual = DynamicViewXmlGenerator.DateToString(date);
        Assert.assertEquals(date.toString(), actual);
    }

    @Test
    public void testDateToString1918(){
        Date date = Date.valueOf(LocalDate.of(1918, 5, 29));
        String actual = DynamicViewXmlGenerator.DateToString(date);
        Assert.assertEquals(date.toString(), actual);
    }

    @Test
    public void testDateToString1947(){
        Date date = Date.valueOf(LocalDate.of(1947, 9, 17));
        String actual = DynamicViewXmlGenerator.DateToString(date);
        Assert.assertEquals(date.toString(), actual);
    }
}
