/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import com.trifork.stamdata.WSDLFetcher;

public class WSDLFetcherImpl implements WSDLFetcher {
	
    private final Map<String, byte[]> resources = new HashMap<>();
    private final List<String> validFiles = new ArrayList<String>(){{
        add("DetGodeCprOpslag.wsdl");
        add("MEDCOM_CprService.xsd");
        add("StamdataPersonLookup.wsdl");
        add("stamdata_cpr_request.xsd");
        add(".xsd");
    }};
	
	@Override
	public Optional<byte[]> get(String path) throws IOException {
		return getResource(path);
	}

    private Optional<byte[]> getResource(String path) throws IOException {
        if (validFiles.stream().noneMatch(path::endsWith)) {
            return Optional.empty();
        }

        if (!resources.containsKey(path)) {
            try (InputStream is = getClass().getResourceAsStream("/" + path)) {
                if (is != null) {
                    try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream()) {
                        byte[] buffer = new byte[256];
                        int bytesRead;
                        while ((bytesRead = is.read(buffer)) != -1) {
                            outputStream.write(buffer, 0, bytesRead);
                        }
                        resources.put(path, outputStream.toByteArray());
                    }
                }
            }
        }
        return Optional.ofNullable(resources.get(path));
    }

}
