/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.mapping;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Properties;

import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;

public class CountryMapper {
	
	private static final Logger logger = Logger.getLogger(CountryMapper.class);

	public String toCountry(String countryCode) {
		String country = countryCodeToCountry.getProperty(countryCode.toUpperCase());
		if (country == null) {
			logger.error("Unknown countryCode: " + countryCode + " setting country to countryCode.");
			country = countryCode;
		}

		return country;
	}

	private static Properties countryCodeToCountry = new Properties();
	static {
		ClassLoader classLoader = MunicipalityMapper.class.getClassLoader();
		InputStream isCountryMappingDataInternal = classLoader.getResourceAsStream("country-mapper-internal.properties");
		InputStream isCountryMappingDataExternal = classLoader.getResourceAsStream("country-mapper-external.properties");
		InputStreamReader countryMappingDataUtf8Internal = null;
		InputStreamReader countryMappingDataUtf8External = null;

		try {
			countryMappingDataUtf8Internal = new InputStreamReader(isCountryMappingDataInternal, "UTF-8");
			countryCodeToCountry.load(countryMappingDataUtf8Internal);
			if (isCountryMappingDataExternal != null) {
				countryMappingDataUtf8External = new InputStreamReader(isCountryMappingDataExternal, "UTF-8");
				countryCodeToCountry.load(countryMappingDataUtf8External);
			}
		}
		catch (IOException e) {
			throw new RuntimeException("Problem loading the country_mapper.properties file", e);
		}
		finally {
			IOUtils.closeQuietly(isCountryMappingDataInternal);
			IOUtils.closeQuietly(countryMappingDataUtf8Internal);
			IOUtils.closeQuietly(isCountryMappingDataExternal);
			IOUtils.closeQuietly(countryMappingDataUtf8External);
		}
	}

}
