/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.pvit;

import dk.nsi._2011._09._23.stamdatacpr.CivilRegistrationNumberListPersonQueryType;
import dk.nsi._2011._09._23.stamdatacpr.NamePersonQueryType;
import dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType;
import dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType;
import dk.nsi._2020._10._01.stamdatacpr.PersonSearchRequestType;
import dk.nsi.stamdata.cpr.mapping.PersonMapper;
import dk.nsi.stamdata.cpr.pvit.v101.StamdataPersonResponseFinder;
import dk.sds.nsp.audit.Audit;
import dk.sds.nsp.audit.AuditBuilder;
import dk.sdsd.nsp.slalog.api.SLALogItem;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.XMLGregorianCalendar;
import java.sql.SQLException;

public class StamdataLookupCommon {

    protected final SLALogItem slaLogItem;
    
    public StamdataLookupCommon(SLALogItem slaLogItem)
    {
        this.slaLogItem = slaLogItem;
    }

    static public PersonLookupResponseType personDetails(PersonLookupRequestType request, String clientCVR, StamdataPersonResponseFinder stamdataPersonResponseFinder, String cprForFiltering) throws SQLException, DatatypeConfigurationException {

        if (request.getCivilRegistrationNumberPersonQuery() != null) {
            logRequestedValue(stamdataPersonResponseFinder.getAuditlogText(), "cpr", request.getCivilRegistrationNumberPersonQuery());
            return stamdataPersonResponseFinder.answerCprRequest(clientCVR, request.getCivilRegistrationNumberPersonQuery());
        }

        if (request.getCivilRegistrationNumberListPersonQuery() != null) {
            logRequestedValue(stamdataPersonResponseFinder.getAuditlogText(), "cpr list", request.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber().toString());
            return stamdataPersonResponseFinder.answerCivilRegistrationNumberListPersonRequest(clientCVR, request.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber(), cprForFiltering);
        }
        if (request.getBirthDatePersonQuery() != null) {
            logRequestedValue(stamdataPersonResponseFinder.getAuditlogText(), "birthdate", request.getBirthDatePersonQuery());
            return stamdataPersonResponseFinder.answerBirthDatePersonRequest(clientCVR, request.getBirthDatePersonQuery(), cprForFiltering);
        }
        if (request.getNamePersonQuery() != null) {
            logRequestedValue(stamdataPersonResponseFinder.getAuditlogText(), "name", (request.getNamePersonQuery().getPersonGivenName()+" "+request.getNamePersonQuery().getPersonMiddleName()+" "+request.getNamePersonQuery().getPersonSurnameName()).replaceAll("  ", " "));
            return stamdataPersonResponseFinder.answerNamePersonRequest(clientCVR, request.getNamePersonQuery(), cprForFiltering);
        }
        throw new AssertionError("Unreachable point: exactly one of the previous clauses is true");
    }

    static public PersonLookupResponseType personDetails(PersonSearchRequestType request, String clientCVR, StamdataPersonResponseFinderCommon stamdataPersonResponseFinder, String cprForFiltering) throws SQLException, DatatypeConfigurationException {
        if (request.getBirthDatePersonQuery() != null) {
            logRequestedValue(stamdataPersonResponseFinder.getAuditlogText(), "birthdate", request.getBirthDatePersonQuery());
            return stamdataPersonResponseFinder.answerBirthDatePersonRequest(clientCVR, request.getBirthDatePersonQuery(), PersonMapper.CPRProtectionLevel.DoNotCensorCPR, cprForFiltering);
        }
        if (request.getNamePersonQuery() != null) {
            logRequestedValue(stamdataPersonResponseFinder.getAuditlogText(), "name", (request.getNamePersonQuery().getPersonGivenName()+" "+request.getNamePersonQuery().getPersonMiddleName()+" "+request.getNamePersonQuery().getPersonSurnameName()).replaceAll("  ", " "));
            return stamdataPersonResponseFinder.answerNamePersonRequest(clientCVR, request.getNamePersonQuery(), PersonMapper.CPRProtectionLevel.DoNotCensorCPR, cprForFiltering);
        }
        throw new AssertionError("Unreachable point: exactly one of the previous clauses is true");
    }

    public static void logRequestedValue(String context, String key, String cpr) {
        AuditBuilder.TypeOfInformation informationType = AuditBuilder.TypeOfInformation.SensitivePersonalInformation;
        String component = "SDM";

        AuditBuilder auditBuilder = Audit.createAuditBuilder();
        auditBuilder.addAuditInformation(component, context, informationType, key, cpr);
    }
    public static void logRequestedValue(String context, String key, NamePersonQueryType name) {
        logRequestedValue(context, key, name.toString());
    }

    public static void logRequestedValue(String context, String key, XMLGregorianCalendar birthDate) {
        logRequestedValue(context, key, birthDate.toString());
    }

    public static void logRequestedValue(String context, String key, CivilRegistrationNumberListPersonQueryType cprList) {
        logRequestedValue(context, key, cprList.toString());
    }
}
