/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.pvit.v101;

import java.sql.SQLException;
import java.util.List;

import javax.inject.Inject;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.XMLGregorianCalendar;

import com.trifork.stamdata.models.PersonVO;

import dk.nsi._2011._09._23.stamdatacpr.NamePersonQueryType;
import dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType;
import dk.nsi.stamdata.cpr.dao.CprDAO;
import dk.nsi.stamdata.cpr.mapping.PersonMapper.CPRProtectionLevel;
import dk.nsi.stamdata.cpr.mapping.v102.PersonMapper;
import dk.nsi.stamdata.cpr.pvit.StamdataPersonResponseFinderCommon;
import dk.nsi.stamdata.cpr.util.UnreliableHandler;
import oio.medcom.cprservice._1_0.PersonInformationStructureType;

public class StamdataPersonResponseFinder extends StamdataPersonResponseFinderCommon {
    @Inject
    public StamdataPersonResponseFinder(CprDAO cprDao, PersonMapper personMapper, UnreliableHandler unreliableHandler) {
        super(cprDao, personMapper, unreliableHandler);
    }

    public PersonInformationStructureType mapPerson(PersonVO person, CPRProtectionLevel cprProtection) {
        return getPersonMapper().map(person, cprProtection);
    }

    public PersonLookupResponseType answerCprRequest(String cvr, String cpr) throws SQLException, DatatypeConfigurationException {
        return answerCprRequest(cvr, cpr, CPRProtectionLevel.DoNotCensorCPR);
    }

    public PersonLookupResponseType answerCivilRegistrationNumberListPersonRequest(String cvr, List<String> civilRegistrationNumberList, String cprForFiltering) throws SQLException {
        return answerCivilRegistrationNumberListPersonRequest(cvr, civilRegistrationNumberList, CPRProtectionLevel.DoNotCensorCPR, cprForFiltering);
    }

    public PersonLookupResponseType answerBirthDatePersonRequest(String cvr, XMLGregorianCalendar birthDate, String cprForFiltering) throws SQLException, DatatypeConfigurationException {
        return answerBirthDatePersonRequest(cvr, birthDate, CPRProtectionLevel.CensorCPR, cprForFiltering);
    }

    public PersonLookupResponseType answerNamePersonRequest(String cvr, NamePersonQueryType namePerson, String cprForFiltering) throws SQLException, DatatypeConfigurationException {
        return answerNamePersonRequest(cvr, namePerson, CPRProtectionLevel.CensorCPR, cprForFiltering);
    }
}
