/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr;

import dk.nsi.stamdata.testing.SecurityContextTestHelper;
import dk.nsi.stamdata.testing.UndertowSlaLogParameters;
import dk.nsi.stamdata.testing.UndertowJaxWsTestServer;
import org.junit.After;
import org.junit.Before;

import javax.inject.Inject;
import javax.inject.Named;
import javax.servlet.ServletException;
import java.io.IOException;


public class AbstractPersonLookupTest {
    private UndertowJaxWsTestServer undertowTestServer;

    @Inject
    @Named("run.undertow")
    private boolean runUndertow;

    @Inject
    @Named("sces.endpoint")
    private String scesEndpoint;

    @Before
    public void setupUndertow() throws ServletException {
        if(runUndertow) {
            undertowTestServer = new UndertowJaxWsTestServer(8100, ComponentController.class, new UndertowSlaLogParameters("Stamdata CPR service", "stamdata-cpr"));
            undertowTestServer.start();
            SecurityContextTestHelper.setContextUserHealthcareProfessionalUser();
        }
    }

    @After
    public void tearDown() throws IOException {
        SecurityContextTestHelper.resetContextOrganisation();
        SecurityContextTestHelper.setContextUserHealthcareProfessionalUser();
        SecurityContextTestHelper.setIdCardValid();
        if(undertowTestServer != null) {
            undertowTestServer.close();
        }
    }

    protected String getScesEndpoint() {
        return scesEndpoint;
    }
}
