/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.medcom;

import static junit.framework.Assert.assertNull;
import static org.junit.Assert.assertEquals;

import java.net.URL;
import java.sql.Connection;
import java.sql.Statement;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPFault;
import javax.xml.ws.soap.SOAPFaultException;

import org.joda.time.DateTime;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.trifork.stamdata.models.ForaeldreMyndighedRelationVO.FMRTypeKode;
import com.trifork.stamdata.models.PersonVO;
import com.trifork.stamdata.models.SikredeVO;
import com.trifork.stamdata.models.YderVO;

import dk.nsi.stamdata.cpr.AbstractPersonLookupTest;
import dk.nsi.stamdata.cpr.mapping.CivilRegistrationStatusCodes;
import dk.nsi.stamdata.security.WhitelistService;
import dk.nsi.stamdata.testing.DatabaseTestInitializer;
import dk.nsi.stamdata.testing.Factories;
import dk.nsi.stamdata.testing.RequestHelper;
import dk.nsi.stamdata.testing.SecurityContextTestHelper;
import dk.nsi.stamdata.testing.StamdataTestHelper;
import dk.nsi.stamdata.testing.TestHelper;
import oio.medcom.cprservice._1_0_4_1a.DetGodeCPROpslag;
import oio.medcom.cprservice._1_0_4_1a.DetGodeCPROpslagService;
import oio.medcom.cprservice._1_0_4_1a.GetPersonInformationIn;
import oio.medcom.cprservice._1_0_4_1a.GetPersonInformationOut;
import oio.medcom.cprservice._1_0_4_1a.GetPersonInformationWithHistoricIn;
import oio.medcom.cprservice._1_0_4_1a.GetPersonInformationWithHistoricOut;
import oio.medcom.cprservice._1_0_4_1a.GetPersonWithHealthCareInformationAndHistoricIn;
import oio.medcom.cprservice._1_0_4_1a.GetPersonWithHealthCareInformationAndHistoricOut;
import oio.medcom.cprservice._1_0_4_1a.GetPersonWithHealthCareInformationIn;
import oio.medcom.cprservice._1_0_4_1a.GetPersonWithHealthCareInformationOut;
import oio.medcom.cprservice._1_0_4_1a.PersonCustodyRelationType;
import oio.medcom.cprservice._1_0_4_1a.PersonCustodyStructureType;
import oio.medcom.cprservice._1_0_4_1a.PersonHealthCareInformationStructureType;
import oio.medcom.cprservice._1_0_4_1a.PersonInformationStructureType;
import oio.medcom.cprservice._1_0_4_1a.PersonWardRelationType;
import oio.medcom.cprservice._1_0_4_1a.PersonWardStructureType;
import oio.medcom.cprservice._1_0_4_1a.PersonWithHealthCareInformationStructureType;
import oio.medcom.cprservice._1_0_4_1a.PublicHealthInsuranceGroupIdentifierType;

@Ignore("Skal kun køres vha. de klasser der nedarver herfra")
public class DetGodeCPROpslag1041A extends AbstractPersonLookupTest {

    public static final QName DET_GODE_CPR_OPSLAG_104A_SERVICE = new QName("urn:oio:medcom:cprservice:1.0.4.1a", "DetGodeCPROpslagService");
    public static final String WHITELISTED_CVR = "33257872";

    @Inject
    private Provider<Connection> connectionProvider;

    private DetGodeCPROpslag client;

    protected GetPersonInformationIn request = new GetPersonInformationIn();
    private GetPersonInformationOut response;
    private GetPersonInformationWithHistoricIn requestWithHistoric = new GetPersonInformationWithHistoricIn();
    private GetPersonInformationWithHistoricOut responseWithHistoric;
    private GetPersonWithHealthCareInformationIn healthCareRequest = new GetPersonWithHealthCareInformationIn();
    protected GetPersonWithHealthCareInformationOut healthCareResponse;
    private GetPersonWithHealthCareInformationAndHistoricIn healthCareRequestWithHistoric = new GetPersonWithHealthCareInformationAndHistoricIn();
    private GetPersonWithHealthCareInformationAndHistoricOut healthCareResponseWithHistoric;


    protected StamdataTestHelper testHelper = new StamdataTestHelper();

    TestHelper helper = new TestHelper();
    private DatabaseTestInitializer dbInitializer;

    private Comparator<PersonCustodyStructureType> personCustodyStructureComparator = new Comparator<PersonCustodyStructureType>() {
        public int compare(PersonCustodyStructureType person, PersonCustodyStructureType person1) {
            return person.getPersonCustodyIdentifier().compareTo(person1.getPersonCustodyIdentifier());
        }
    };

    @Inject
    Connection connection;

    @Before
    public void setUpDB() throws Exception
    {
        connection.setAutoCommit(false);
        dbInitializer = new DatabaseTestInitializer(connection);
        dbInitializer.initialize();
        Statement statement = connection.createStatement();
        statement.executeUpdate("INSERT IGNORE INTO whitelist_config SET component_name='"+ WhitelistService.DEFAULT_SERVICE_NAME +"', cvr='"+WHITELISTED_CVR+"'");
        connection.commit();
    }

    @Before
    public void setUp() throws Exception {
        URL wsdlLocation = new URL(String.format("%s/service/DetGodeCPROpslag-1.0.4.1a?wsdl", getScesEndpoint()));
        DetGodeCPROpslagService serviceCatalog =
                new DetGodeCPROpslagService(wsdlLocation, DET_GODE_CPR_OPSLAG_104A_SERVICE);
        client = serviceCatalog.getDetGodeCPROpslag();
        setContextUserCitizenUser();
    }
    protected void setContextUserCitizenUser() {
    	//empty default
    }
    protected void setContextUserCitizenUser(String cpr) {
    	//empty default
    }
    
    private void sendPersonRequest(String cpr) throws Exception {
    	setContextUserCitizenUser(cpr);
    	helper.enableAuditlogging(true);
    	helper.redirectSystemOutToStream();
        request.setPersonCivilRegistrationIdentifier(cpr);
        try {
            performRequest();
        } catch (Exception e) {
            helper.enableAuditlogging(false);
            helper.resetSystemOut();
            throw e;
        }
        helper.enableAuditlogging(false);
        helper.resetSystemOut();
        String actualLog = helper.getSystemLogs("DEVELOPMENT AUDIT: ");
        String expectedLog = "DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.0.4.1a getPersonInformation\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\"" + request.getPersonCivilRegistrationIdentifier() + "\"}";
        assertEquals(expectedLog, actualLog);
    }

    private void sendPersonRequestWithHistoric(String cpr) throws Exception {
    	setContextUserCitizenUser(cpr);
        helper.enableAuditlogging(true);
        helper.redirectSystemOutToStream();
        requestWithHistoric.setPersonCivilRegistrationIdentifier(cpr);
        try {
            performRequestWithHistoric();
        } catch (Exception e) {
            helper.enableAuditlogging(false);
            helper.resetSystemOut();
            throw e;
        }
        helper.enableAuditlogging(false);
        helper.resetSystemOut();
        String actualLog = helper.getSystemLogs("DEVELOPMENT AUDIT: ");
        String expectedLog = "DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.0.4.1a getPersonInformationWithHistoric\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\"" + requestWithHistoric.getPersonCivilRegistrationIdentifier() + "\"}";
        assertEquals(expectedLog, actualLog);
    }

    private void sendHealthCareRequest() throws Exception {
        helper.enableAuditlogging(true);
        helper.redirectSystemOutToStream();
        testHelper.prepareTestdata(connection);
        try {
            healthCareResponse = client.getPersonWithHealthCareInformation(healthCareRequest);
        } catch (Exception e) {
            helper.enableAuditlogging(false);
            helper.resetSystemOut();
            throw e;
        }
        helper.enableAuditlogging(false);
        helper.resetSystemOut();

        String actualLog = helper.getSystemLogs("DEVELOPMENT AUDIT: ");
        String expectedLog = "DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.0.4.1a getPersonWithHealthcareInformation\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+healthCareRequest.getPersonCivilRegistrationIdentifier()+"\"}";
        assertEquals(expectedLog, actualLog);
    }

    protected void sendHealthCareRequest(String cpr) throws Exception {
    	setContextUserCitizenUser(cpr);
        healthCareRequest.setPersonCivilRegistrationIdentifier(cpr);
        sendHealthCareRequest();
    }

    private void sendHealthCareRequestWithHistoric() throws Exception {
        helper.enableAuditlogging(true);
        helper.redirectSystemOutToStream();
        testHelper.prepareTestdata(connection);
        try {
            healthCareResponseWithHistoric = client.getPersonWithHealthCareInformationAndHistoric(healthCareRequestWithHistoric);
        } catch (Exception e) {
            helper.enableAuditlogging(false);
            helper.resetSystemOut();
            throw e;
        }
        helper.enableAuditlogging(false);
        helper.resetSystemOut();

        String actualLog = helper.getSystemLogs("DEVELOPMENT AUDIT: ");
        String expectedLog = "DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.0.4.1a getPersonWithHealthcareInformationAndHistoric\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\""+healthCareRequestWithHistoric.getPersonCivilRegistrationIdentifier()+"\"}";
        assertEquals(expectedLog, actualLog);
    }

    private void sendHealthCareRequestWithHistoric(String cpr) throws Exception {
    	setContextUserCitizenUser(cpr);
        healthCareRequestWithHistoric.setPersonCivilRegistrationIdentifier(cpr);
        sendHealthCareRequestWithHistoric();
    }

    protected void performRequest() {
        response = client.getPersonInformation(request);
    }

    private void performRequestWithHistoric() {
        responseWithHistoric = client.getPersonInformationWithHistoric(requestWithHistoric);
    }

    @Test
    public void requestForPersonInformation() throws Exception {
        PersonVO person = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR);
        testHelper.createPersonFamilie(person, personMor, personFar);

        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createPersonFamilie(personBarn1, null, person);

        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createPersonFamilie(personBarn2, null, person);

        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person);
        testHelper.createVaerge(personBarn4, person);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());

        AssertResponse1041a.assertPersonInformationStructure(response.getPersonInformationStructure());
    }

    @Test
    public void requestForPersonInformationWithProtection() throws Exception {
        PersonVO person = testHelper.createPerson();
        person.setNavnebeskyttelsestartdato(testHelper.formatSimpleDate("2000-01-01"));
        person.setNavnebeskyttelseslettedato(testHelper.formatSimpleDate("2050-01-01"));
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR);
        testHelper.createPersonFamilie(person, personMor, personFar);

        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createPersonFamilie(personBarn1, null, person);

        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createPersonFamilie(personBarn2, null, person);

        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person);
        testHelper.createVaerge(personBarn4, person);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());

        AssertResponse1041a.assertPersonInformationStructureProtected(response.getPersonInformationStructure());
    }

    @Test
    public void requestForPersonWithHealthCareInformation() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        PersonVO person2 = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person1, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.FAR);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.MOR);
        testHelper.createPersonFamilie(person1, personMor, personFar);

        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createPersonFamilie(personBarn1, null, person1);

        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createPersonFamilie(personBarn2, null, person1);

        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person1);
        testHelper.createVaerge(personBarn4, person1);
        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("4321");
        SikredeVO sikrede1 = testHelper.createSikredeWithFrontierInfo(person1, yder1, "1", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikrede1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yder2, "4", new DateTime(2011, 10, 10, 0, 0));
        // Having multiple persons ensures that we are selecting the
        // right one.

        sendHealthCareRequest(person1.getCpr());

        PersonWithHealthCareInformationStructureType personWithHealthCareInformationStructure = healthCareResponse.getPersonWithHealthCareInformationStructure();
        AssertResponse1041a.assertPersonWithHealthCareInformationStructure(personWithHealthCareInformationStructure);
    }

    @Test
    public void requestForPersonWithHealthCareInformationWithProtection() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        person1.setNavnebeskyttelsestartdato(testHelper.formatSimpleDate("2000-01-01"));
        person1.setNavnebeskyttelseslettedato(testHelper.formatSimpleDate("2050-01-01"));
        PersonVO person2 = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createKvinde(StamdataTestHelper.CPR_SPOUSE);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personBarn1 = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personBarn2 = testHelper.createPerson(StamdataTestHelper.CPR_BARN2);
        PersonVO personBarn3 = testHelper.createPerson(StamdataTestHelper.CPR_BARN3);
        PersonVO personBarn4 = testHelper.createPerson(StamdataTestHelper.CPR_BARN4);
        testHelper.createSpouse(person1, personSpouse);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.FAR);
        testHelper.createForaeldreMyndighedRelation(person1, FMRTypeKode.MOR);
        testHelper.createPersonFamilie(person1, personMor, personFar);

        testHelper.createForaeldreMyndighedRelation(personBarn1, FMRTypeKode.FAR);
        testHelper.createPersonFamilie(personBarn1, null, person1);

        testHelper.createForaeldreMyndighedRelation(personBarn2, FMRTypeKode.MOR);
        testHelper.createPersonFamilie(personBarn2, null, person1);

        testHelper.createForaeldreMyndighedRelation(personBarn3, FMRTypeKode.AI1, person1);
        testHelper.createVaerge(personBarn4, person1);
        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("4321");
        SikredeVO sikrede1 = testHelper.createSikredeWithFrontierInfo(person1, yder1, "1", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikrede1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yder2, "4", new DateTime(2011, 10, 10, 0, 0));
        // Having multiple persons ensures that we are selecting the
        // right one.

        sendHealthCareRequest(person1.getCpr());

        PersonWithHealthCareInformationStructureType personWithHealthCareInformationStructure = healthCareResponse.getPersonWithHealthCareInformationStructure();
        AssertResponse1041a.assertPersonWithHealthCareInformationStructureProtected(personWithHealthCareInformationStructure);
    }

    @Test
    public void requestWithGroupCode0() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        PersonVO person2 = testHelper.createPerson();
        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("4321");
        SikredeVO sikrede1 = testHelper.createSikredeWithFrontierInfo(person1, yder1, "0", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikrede1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yder2, "4", new DateTime(2011, 10, 10, 0, 0));
        // Having multiple persons ensures that we are selecting the
        // right one.

        sendHealthCareRequest(person1.getCpr());

        PersonWithHealthCareInformationStructureType personWithHealthCareInformationStructure = healthCareResponse.getPersonWithHealthCareInformationStructure();
        PersonHealthCareInformationStructureType personHealthCareInformationStructure = personWithHealthCareInformationStructure.getPersonHealthCareInformationStructure();
        AssertResponse1041a.assertPersonHealthCareInformationStructure(personHealthCareInformationStructure, PublicHealthInsuranceGroupIdentifierType.SYGESIKRINGSGRUPPE_0);
    }

    @Test
    public void requestForSeperatedPerson() throws Exception {
        PersonVO person = testHelper.createPerson();
        PersonVO personSpouse = testHelper.createPerson(StamdataTestHelper.CPR_SPOUSE);
        testHelper.createSeperated(person, personSpouse);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();

        assertNull(personInformationStructure.getPersonSpouseStructure());
    }

    @Test
    public void requestForUnmarriedPerson() throws Exception {
        PersonVO person = testHelper.createPerson();
        testHelper.createUnmarried(person);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();

        assertNull(personInformationStructure.getPersonSpouseStructure());
    }

    @Ignore("SDS-5361 - skal denne exception testes i integrationstest?")
    @Test
    public void testSignatureInvalidModelBuildException() throws Exception {

        String fileName = "src/test/resources/data/validXml.xml";
        String xml = RequestHelper.readFile(fileName);
        String replication_endpoint = String.format("%s/service/DetGodeCPROpslag-1.0.4", getScesEndpoint());

        SecurityContextTestHelper.setIdCardExpired();
        String responseXml = RequestHelper.sendRequest(replication_endpoint, "http://rep.oio.dk/medcom.sundcom.dk/xml/wsdl/2007/06/28/getPersonInformation", xml);

        String faultstring = RequestHelper.getFaultstring(responseXml);
        assertEquals(faultstring, "Cannot find dispatch method for {urn:oio:medcom:cprservice:1.0.2}getPersonInformationIn");
        
        //After
        SecurityContextTestHelper.setIdCardValid();
    }

    @Test
    public void requestForCustodyAndenIndehaver() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personFar = testHelper.createMand(StamdataTestHelper.CPR_FAR);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);

        testHelper.createPersonFamilie(person, null, personFar);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.FAR, personFar);

        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.AI1, personMor);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(StamdataTestHelper.CPR_BARN1);

        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();
        assertEquals(2,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        AssertResponse1041a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_FAR, PersonCustodyRelationType.FAR);
        personCustody = personCustodyStructure.get(1);
        AssertResponse1041a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MOR, PersonCustodyRelationType.ANDEN);

        sendPersonRequest(StamdataTestHelper.CPR_FAR);

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        PersonWardStructureType ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MOR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
    }

    @Test
    public void requestForCustodyMedmor() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personMedmor = testHelper.createKvinde(StamdataTestHelper.CPR_MEDMOR);

        testHelper.createPersonFamilie(person, personMor, null);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR, personMor);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.AI1, personMedmor);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(StamdataTestHelper.CPR_BARN1);

        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();
        Collections.sort(personCustodyStructure, personCustodyStructureComparator);
        assertEquals(2,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        AssertResponse1041a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MOR, PersonCustodyRelationType.MOR);
        personCustody = personCustodyStructure.get(1);
        AssertResponse1041a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MEDMOR, PersonCustodyRelationType.ANDEN);

        sendPersonRequest(StamdataTestHelper.CPR_MOR);

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        PersonWardStructureType ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MEDMOR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
    }

    @Test
    public void requestForCustodyMedfar() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personMor = testHelper.createKvinde(StamdataTestHelper.CPR_MOR);
        PersonVO personMedfar = testHelper.createMand(StamdataTestHelper.CPR_MEDFAR);

        testHelper.createPersonFamilie(person, personMor, null);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.MOR, personMor);
        testHelper.createForaeldreMyndighedRelation(person, FMRTypeKode.AI1, personMedfar);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(StamdataTestHelper.CPR_BARN1);

        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();
        Collections.sort(personCustodyStructure, personCustodyStructureComparator);
        assertEquals(2,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        AssertResponse1041a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MOR, PersonCustodyRelationType.MOR);
        personCustody = personCustodyStructure.get(1);
        AssertResponse1041a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_MEDFAR, PersonCustodyRelationType.ANDEN);

        sendPersonRequest(StamdataTestHelper.CPR_MOR);

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        PersonWardStructureType ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());

        sendPersonRequest(StamdataTestHelper.CPR_MEDFAR);

        personInformationStructure = response.getPersonInformationStructure();
        personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.BARN,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
    }

    @Test
    public void requestForVaerge() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personVaerge = testHelper.createPerson(StamdataTestHelper.CPR_VAERGE);
        testHelper.createVaerge(person, personVaerge);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();

        assertEquals(1,personCustodyStructure.size());
        PersonCustodyStructureType personCustody = personCustodyStructure.get(0);
        AssertResponse1041a.assertpersonCustody(personCustody, StamdataTestHelper.CPR_VAERGE, PersonCustodyRelationType.VAERGE);

        sendPersonRequest(personVaerge.getCpr());

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(1,personWardStructure.size());
        PersonWardStructureType ward = personWardStructure.get(0);
        assertEquals(PersonWardRelationType.MYNDLING,ward.getPersonWardRelation());
        assertEquals(StamdataTestHelper.CPR_BARN1, ward.getPersonWardIdentifier());
    }

    @Test
    public void requestForVaergeUdenCPR() throws Exception {
        PersonVO person = testHelper.createPerson(StamdataTestHelper.CPR_BARN1);
        PersonVO personVaerge = testHelper.createPerson(StamdataTestHelper.CPR_VAERGE);
        testHelper.createVaerge(person, null);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();
        List<PersonCustodyStructureType> personCustodyStructure = personInformationStructure.getPersonCustodyStructure();

        assertEquals(0,personCustodyStructure.size());

        sendPersonRequest(personVaerge.getCpr());

        personInformationStructure = response.getPersonInformationStructure();
        List<PersonWardStructureType> personWardStructure = personInformationStructure.getPersonWardStructure();
        assertEquals(0,personWardStructure.size());
    }

    @Test
    public void requestForDeadPersonInformation() throws Exception {
        PersonVO person = testHelper.createPerson();
        person.setStatus(CivilRegistrationStatusCodes.DEAD);

        testHelper.prepareTestdata(connection);
        sendPersonRequest(person.getCpr());
        PersonInformationStructureType personInformationStructure = response.getPersonInformationStructure();

        AssertResponse1041a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getCurrentPersonCivilRegistrationIdentifier());
        AssertResponse1041a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getPriorPersonCivilRegistrationIdentifier());
        AssertResponse1041a.assertRegularCPRPerson(personInformationStructure.getRegularCPRPerson(), false, CivilRegistrationStatusCodes.DEAD);
        AssertResponse1041a.assertPersonAddressStructure(personInformationStructure.getPersonAddressStructure(),false, true);
    }

    @Test
    public void requestForDeadPersonInformationWithHistoric() throws Exception {
        PersonVO person = testHelper.createPerson();
        person.setStatus(CivilRegistrationStatusCodes.DEAD);

        testHelper.prepareTestdata(connection);
        sendPersonRequestWithHistoric(person.getCpr());
        PersonInformationStructureType personInformationStructure = responseWithHistoric.getPersonInformationStructure();

        AssertResponse1041a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getCurrentPersonCivilRegistrationIdentifier());
        AssertResponse1041a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getPriorPersonCivilRegistrationIdentifier());
        AssertResponse1041a.assertRegularCPRPerson(personInformationStructure.getRegularCPRPerson(), false, CivilRegistrationStatusCodes.DEAD);
        AssertResponse1041a.assertPersonAddressStructure(personInformationStructure.getPersonAddressStructure(),false, false);
    }

    @Test
    public void requestForDeadPersonInformationWithHealthCareInformation() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        person1.setStatus(CivilRegistrationStatusCodes.DEAD);
        PersonVO person2 = testHelper.createPerson();

        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("4321");
        SikredeVO sikrede1 = testHelper.createSikredeWithFrontierInfo(person1, yder1, "1", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikrede1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yder2, "4", new DateTime(2011, 10, 10, 0, 0));

        sendHealthCareRequest(person1.getCpr());
        PersonInformationStructureType personInformationStructure = healthCareResponse.getPersonWithHealthCareInformationStructure().getPersonInformationStructure();

        AssertResponse1041a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getCurrentPersonCivilRegistrationIdentifier());
        AssertResponse1041a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getPriorPersonCivilRegistrationIdentifier());
        AssertResponse1041a.assertRegularCPRPerson(personInformationStructure.getRegularCPRPerson(), false, CivilRegistrationStatusCodes.DEAD);
        AssertResponse1041a.assertPersonAddressStructure(personInformationStructure.getPersonAddressStructure(),false, true);
    }

    @Test
    public void requestForDeadPersonInformationWithHealthCareInformationAnndHistoric() throws Exception {
        PersonVO person1 = testHelper.createPerson();
        person1.setStatus(CivilRegistrationStatusCodes.DEAD);
        PersonVO person2 = testHelper.createPerson();

        YderVO yder1 = testHelper.createYder("1234");
        YderVO yder2 = testHelper.createYder("4321");
        SikredeVO sikrede1 = testHelper.createSikredeWithFrontierInfo(person1, yder1, "1", new DateTime(Factories.YESTERDAY),new DateTime(Factories.YEAR_2000),true);
        sikrede1.setSYdernrGl("4321");
        testHelper.createSikrede(person2, yder2, "4", new DateTime(2011, 10, 10, 0, 0));

        sendHealthCareRequestWithHistoric(person1.getCpr());
        PersonInformationStructureType personInformationStructure = healthCareResponseWithHistoric.getPersonWithHealthCareInformationStructure().getPersonInformationStructure();

        AssertResponse1041a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getCurrentPersonCivilRegistrationIdentifier());
        AssertResponse1041a.assertPersonCivilRegistrationIdentifier(personInformationStructure.getPriorPersonCivilRegistrationIdentifier());
        AssertResponse1041a.assertRegularCPRPerson(personInformationStructure.getRegularCPRPerson(), false, CivilRegistrationStatusCodes.DEAD);
        AssertResponse1041a.assertPersonAddressStructure(personInformationStructure.getPersonAddressStructure(),false, false);
    }

    @Test
    public void requestForPersonWithoutStatus() throws Exception {
        PersonVO person = testHelper.createPerson();
        person.setStatus(null);
        testHelper.prepareTestdata(connection);
        try {
            sendPersonRequest(person.getCpr());
        } catch (Exception e) {
            SOAPFault fault = ((SOAPFaultException)e).getFault();
            Assert.assertTrue(fault.getFirstChild().getFirstChild().toString().contains("Client"));
        }
    }
    
    @Test
    public void requestFrontierPersonWithSBSStatsborgerskabNullAndSBSStatsborgerskabKodeAvailable() throws Exception {
        PersonVO person1 = testHelper.createPerson();

        YderVO yder1 = testHelper.createYder("1234");

        DateTime fromDate = new DateTime(2011, 10, 10, 0, 0);

        SikredeVO sikredeVO = testHelper.createSikredeWithFrontierInfo(person1, yder1, "9", fromDate, fromDate,true);
        sikredeVO.setSBSStatsborgerskab(null);

        sendHealthCareRequest(person1.getCpr());
        
        Assert.assertNotNull(healthCareResponse.getPersonWithHealthCareInformationStructure().getFrontierInformationStructure().getFrontierCitizenshipStructure());
        Assert.assertEquals("GB",healthCareResponse.getPersonWithHealthCareInformationStructure().getFrontierInformationStructure().getFrontierCitizenshipStructure().getFrontierCitizenshipCode());
        Assert.assertEquals("United Kingdom of Great Britain and Northern Ireland (the)",healthCareResponse.getPersonWithHealthCareInformationStructure().getFrontierInformationStructure().getFrontierCitizenshipStructure().getFrontierCitizenship());

    } //Andre kombinationer af SBSStatsborgerskab ikke-udfyldt findes i DetGodeCPROpslagIntegrationTest som generel test, da løsningen er lavet end. Ovenstående test er lavet for at tjekke at denne version har ændringen.

}
