/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.medcom;

import static org.junit.Assert.assertEquals;

import org.junit.Test;
import org.junit.runner.RunWith;

import com.trifork.stamdata.models.PersonVO;

import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.testing.SecurityContextTestHelper;

@RunWith(GuiceTestRunner.class)
public class DetGodeCPROpslag1041AidwsTest extends DetGodeCPROpslag1041A {
	
	//Genbrug af testene fra DetGodeCPROpslag1041. Specifikke idws test laves nedenfor.

	@Override
    protected void setContextUserCitizenUser() {
		SecurityContextTestHelper.setContextUserCitizenUser();
    }
	@Override
    protected void setContextUserCitizenUser(String cpr) {
		SecurityContextTestHelper.setContextUserCitizenUser(cpr);
    }

    @Test
    public void requestForOtherPerson() throws Exception {
        PersonVO person = testHelper.createPerson();

        try {
            sendPersonRequestForOtherCpr(person.getCpr(), "0103952595");
        } catch (Exception e) {
            assertEquals("Adgang til CPR-nummeret ikke tilladt", e.getMessage());
        }
    }

    private void sendPersonRequestForOtherCpr(String requestCpr, String SecurityContextCpr) throws Exception {
    	setContextUserCitizenUser(SecurityContextCpr);
        helper.enableAuditlogging(true);
        helper.redirectSystemOutToStream();
        request.setPersonCivilRegistrationIdentifier(requestCpr);
        try {
            performRequest();
        } catch (Exception e) {
            helper.enableAuditlogging(false);
            helper.resetSystemOut();
            throw e;
        }
        helper.enableAuditlogging(false);
        helper.resetSystemOut();
        String actualLog = helper.getSystemLogs("DEVELOPMENT AUDIT: ");
        String expectedLog = "DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.0.4.1a getPersonInformation\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr\",\"value\":\"" + request.getPersonCivilRegistrationIdentifier() + "\"}";
        assertEquals(expectedLog, actualLog);
    }
    
}
