/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.pvit;

import static org.junit.Assert.assertEquals;

import org.junit.Test;
import org.junit.runner.RunWith;

import com.trifork.stamdata.models.PersonVO;

import dk.nsi._2011._09._23.stamdatacpr.CivilRegistrationNumberListPersonQueryType;
import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.testing.SecurityContextTestHelper;

@RunWith(GuiceTestRunner.class)
public class StamdataPersonLookupDGWS120Test extends StamdataPersonLookup120 {
	
	//Genbrug af testene fra StamdataPersonLookup120. Specifikke dgws test laves nedenfor.

	@Override
	protected void setContextOrganisation() {
		//Når dgws sættes følgende
    	SecurityContextTestHelper.setContextOrganisation(WHITELISTED_CVR);
    }
	
	/**
	 * Test med 2 i resultat kun relevant ved DGWS da cpr listen indeholder mere end eet cpr nummer, IDWS (borger) tillader kun returnering af egne data - dvs. eet cpr nummer 
	 */
	@Test
	public void requestCPRprotectedWithoutConsent2() throws Exception {

		PersonVO person = addPerson("0103952595");
		addPerson("0103952596");
		setContextUserCitizenUser(person.getCpr());

		CivilRegistrationNumberListPersonQueryType cprList = new CivilRegistrationNumberListPersonQueryType();
		cprList.getCivilRegistrationNumber().add("0103952595");
		cprList.getCivilRegistrationNumber().add("0103952596");
		getRequest.setCivilRegistrationNumberListPersonQuery(cprList);
		prepareDatabaseAndSendRequest();
		assertResponseWithProtectedAddress(0);
		assertResponseWithProtectedAddress(1);
		assertCpr(person);

		String[] logs = helper.getSystemLogs();
		assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.2.0 getPersonDetails\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr list\",\"value\":\""+getRequest.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber()+ "\"}", logs[0]);
  }

	/**
	 * Test med 2 i resultat kun relevant ved DGWS da cpr listen indeholder mere end eet cpr nummer, IDWS (borger) tillader kun returnering af egne data - dvs. eet cpr nummer 
	 */
	@Test
	public void requestCPRprotectedWithConsent2() throws Exception {

		PersonVO person = addPerson("0103952595");
		setContextUserCitizenUser(person.getCpr());
		addPerson("0103952596");

		CivilRegistrationNumberListPersonQueryType cprList = new CivilRegistrationNumberListPersonQueryType();
		cprList.getCivilRegistrationNumber().add("0103952595");
		cprList.getCivilRegistrationNumber().add("0103952596");
		getRequest.setCivilRegistrationNumberListPersonQuery(cprList);
		prepareDatabaseAndSendRequest(RequestType.GET_PERSON_WITH_CONSENT);
		assertResponseWithFullAddress(0);
		assertResponseWithFullAddress(1);
		assertCpr(person);

		String[] logs = helper.getSystemLogs();
		assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.2.0 getPersonDetailsWithConsent\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr list\",\"value\":\""+getRequest.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber()+ "\"}", logs[0]);
	}

}
