/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.cpr.pvit;

import static org.junit.Assert.assertEquals;

import java.util.GregorianCalendar;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.junit.runner.RunWith;

import com.trifork.stamdata.models.PersonVO;

import dk.nsi._2011._09._23.stamdatacpr.CivilRegistrationNumberListPersonQueryType;
import dk.nsi._2011._09._23.stamdatacpr.NamePersonQueryType;
import dk.nsi.stamdata.guice.GuiceTestRunner;
import dk.nsi.stamdata.testing.SDMSecurityContextHelperCitizen;
import dk.nsi.stamdata.testing.SecurityContextTestHelper;
import dk.sds.nsp.security.SecurityProvider;

@RunWith(GuiceTestRunner.class)
public class StamdataPersonLookupIDWS120Test extends StamdataPersonLookup120 {
	
	//Genbrug af testene fra StamdataPersonLookup120. Specifikke idws test laves nedenfor.
	
	@Override
	protected void setContextUserCitizenUser() {
		//Når idws sættes følgende
		SecurityContextTestHelper.setContextUserCitizenUser();
	}
	@Override
	protected void setContextUserCitizenUser(String cpr) {
		//Når idws sættes følgende
		SecurityContextTestHelper.setContextUserCitizenUser(cpr);
	}
	
	@Test
	public void requestCPRprotectedWithoutConsentOtherCpr() throws Exception {
		System.setProperty(SecurityProvider.DEVELOPMENT_PROVIDER_PROPERTY, SDMSecurityContextHelperCitizen.class.getName());
		SecurityContextTestHelper.setContextUserCitizenUser("0123456789");
		SecurityContextTestHelper.setContextCitizen();

		PersonVO person = addPerson("0103952595");
		getRequest.setCivilRegistrationNumberPersonQuery("0103952595");
		try {
			prepareDatabaseAndSendRequest();
		} catch (Exception e) {
			assertEquals("Adgang til CPR-nummeret ikke tilladt", e.getMessage());
		}
	}
	@Rule
	public ExpectedException exceptionRule = ExpectedException.none();

	@Test
    public void requestSearchCprNotOwnData_name() throws Exception {
    	
    	String cpr = "0103952595";
    	String requesterCpr = "0103952596";
    	setContextUserCitizenUser(requesterCpr);
        PersonVO person = addPerson(cpr);

        NamePersonQueryType namePersonQuery = new NamePersonQueryType();
        namePersonQuery.setPersonGivenName("Peter");
        namePersonQuery.setPersonMiddleName("Sigurd");
        namePersonQuery.setPersonSurnameName("Andersen");

        searchRequest.setNamePersonQuery(namePersonQuery);

        prepareDatabaseAndSendRequest(RequestType.SEARCH_PERSON);
        assertResponseSize(0);

    }

    @Test
    public void requestSearchCprNotOwnData_birthDate() throws Exception {

    	String cpr = "0103952595";
    	String requesterCpr = "0103952596";
    	setContextUserCitizenUser(requesterCpr);
        PersonVO person = addPerson(cpr);

        GregorianCalendar cal = new GregorianCalendar();
        cal.setTime(persons.get(0).getFoedselsdato());
        XMLGregorianCalendar birthday = DatatypeFactory.newInstance().newXMLGregorianCalendar(cal);

        searchRequest.setBirthDatePersonQuery(birthday);

        prepareDatabaseAndSendRequest(RequestType.SEARCH_PERSON);
        assertResponseSize(0);
        
    }
    
	/**
	 * Test med 2 i resultat kun relevant ved DGWS da cpr listen indeholder mere end eet cpr nummer, IDWS (borger) tillader kun returnering af egne data - dvs. eet cpr nummer 
	 */
	@Test
	public void requestCPRprotectedWithoutConsent2() throws Exception {

		PersonVO person = addPerson("0103952595");
		addPerson("0103952596");
		setContextUserCitizenUser(person.getCpr());

		CivilRegistrationNumberListPersonQueryType cprList = new CivilRegistrationNumberListPersonQueryType();
		cprList.getCivilRegistrationNumber().add("0103952595");
		cprList.getCivilRegistrationNumber().add("0103952596");
		getRequest.setCivilRegistrationNumberListPersonQuery(cprList);
		prepareDatabaseAndSendRequest();
		assertResponseSize(1);
		assertResponseWithProtectedAddress(0);
		assertCpr(person);

		String[] logs = helper.getSystemLogs();
		assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.2.0 getPersonDetails\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr list\",\"value\":\""+getRequest.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber()+ "\"}", logs[0]);

  }

	/**
	 * Test med 2 i resultat kun relevant ved DGWS da cpr listen indeholder mere end eet cpr nummer, IDWS (borger) tillader kun returnering af egne data - dvs. eet cpr nummer 
	 */
	@Test
	public void requestCPRprotectedWithConsent2() throws Exception {
		
		PersonVO person = addPerson("0103952595");
		setContextUserCitizenUser(person.getCpr());
		addPerson("0103952596");

		CivilRegistrationNumberListPersonQueryType cprList = new CivilRegistrationNumberListPersonQueryType();
		cprList.getCivilRegistrationNumber().add("0103952595");
		cprList.getCivilRegistrationNumber().add("0103952596");
		getRequest.setCivilRegistrationNumberListPersonQuery(cprList);
		prepareDatabaseAndSendRequest(RequestType.GET_PERSON_WITH_CONSENT);
		assertResponseSize(1);
		assertResponseWithFullAddress(0);
		assertCpr(person);

		String[] logs = helper.getSystemLogs();
		assertEquals("DEVELOPMENT AUDIT: {\"component\":\"SDM\",\"context\":\"SCES v1.2.0 getPersonDetailsWithConsent\",\"type\":\"SensitivePersonalInformation\",\"key\":\"cpr list\",\"value\":\""+getRequest.getCivilRegistrationNumberListPersonQuery().getCivilRegistrationNumber()+ "\"}", logs[0]);

	}

	
}
