/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.authorization;

import authorization.AuthorizationResponseType;
import dk.nsi.stamdata.client.AuthorisationClient;
import dk.nsi.stamdata.configuration.NspServiceException;
import dk.nsi.stamdata.configuration.SDMContextConfiguration;
import dk.nsp.test.idp.OrganizationIdentities;
import dk.nsp.test.idp.model.Identity;
import dk.sosi.seal.model.AuthenticationLevel;
import org.junit.*;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.rules.SpringClassRule;
import org.springframework.test.context.junit4.rules.SpringMethodRule;

import javax.xml.bind.JAXBException;


@RunWith(Parameterized.class)
@ContextConfiguration(classes = SDMContextConfiguration.class )
public class AuthorizationIntegrationTest {

    @ClassRule
    public static final SpringClassRule scr = new SpringClassRule();

    @Rule
    public final SpringMethodRule smr = new SpringMethodRule();

    @Parameterized.Parameter(0)
    public AuthenticationLevel authenticationLevel;

    @Value("${saes.endpoint}")
    private String saesEndpoint;

    private Identity identity = OrganizationIdentities.sundhedsdatastyrelsen();

    private AuthorisationClient authorisationClient;

    @Parameterized.Parameters(name = "{0}")
    public static AuthenticationLevel[] data() {
        return new AuthenticationLevel[]{AuthenticationLevel.VOCES_TRUSTED_SYSTEM};
    }

    @Before
    public void setup() {
        authorisationClient = new AuthorisationClient();
    }

    @Test
    public void getPersonWithAuthorization() throws NspServiceException , JAXBException{

        // Given
        String cpr = "0304784567";

        // When
        AuthorizationResponseType reponse = authorisationClient.callAuthorise(cpr, identity, saesEndpoint);

        // Then
        Assert.assertEquals(1, reponse.getAuthorization().size());
        Assert.assertEquals("45RTX", reponse.getAuthorization().get(0).getAuthorizationCode());
        Assert.assertEquals("7170", reponse.getAuthorization().get(0).getEducationCode());
        Assert.assertEquals("Egon", reponse.getFirstName());
        Assert.assertEquals("Olsen", reponse.getLastName());
        Assert.assertEquals("0304784567", reponse.getCpr());
    }

    @Test
    public void getPersonWithoutAuthorization() throws JAXBException, NspServiceException{

        // Given
        String cpr = "0000000000";

        // When
        AuthorizationResponseType reponse = authorisationClient.callAuthorise(cpr, identity, saesEndpoint);

        // Then
        Assert.assertEquals(reponse.getAuthorization().size(), 0);
    }

    @Test
    public void getPersonWithAuthorizationCode() throws JAXBException, NspServiceException{

        // Given
        String authorizationCode = "45RTX";

        // When
        AuthorizationResponseType reponse = authorisationClient.callAuthoriseCode(authorizationCode, identity, saesEndpoint);

        // Then
        Assert.assertEquals(1, reponse.getAuthorization().size());
        Assert.assertEquals("45RTX", reponse.getAuthorization().get(0).getAuthorizationCode());
        Assert.assertEquals("7170", reponse.getAuthorization().get(0).getEducationCode());
        Assert.assertEquals("Egon", reponse.getFirstName());
        Assert.assertEquals("Olsen", reponse.getLastName());
        Assert.assertEquals("0304784567", reponse.getCpr());
    }
}
