/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.client;

import authorization.*;
import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.nsi.stamdata.configuration.NspServiceException;
import dk.nsp.test.idp.model.Identity;
import org.apache.log4j.Logger;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import java.net.URI;
import java.net.URISyntaxException;

public class AuthorisationClient  extends AbstractSDMClient {

    private Logger log = Logger.getLogger(getClass());

    public AuthorizationResponseType callAuthorise(String cpr, Identity identity, String saesEndpoint) throws JAXBException, NspServiceException{
        JAXBContext jaxbContext = JAXBContext.newInstance(AuthorizationRequestType.class, AuthorizationResponseType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class, authorization.Header.class, Security.class, new QName("http://nsi.dk/-/stamdata/3.0", "AuthorizationResponseStructure").getClass());

        // Create request
        AuthorizationRequestType request = new ObjectFactory().createAuthorizationRequestType();
        request.setCpr(cpr);

        JAXBElement<AuthorizationRequestType> requestXml = new ObjectFactory().createAuthorizationRequestStructure(request);
        AuthorizationResponseType authorisationResponse = null;
        try {
            authorisationResponse = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/sdm/Gateway/", new URI(saesEndpoint + "/service/AuthorizationService"), AuthorizationResponseType.class);
        }catch (URISyntaxException e){
            log.debug("SaesEndpoint is not a valid URI");
        }
        return authorisationResponse;
    }

    public AuthorizationResponseType callAuthoriseCode(String authorisationCode, Identity identity, String saesEndpoint) throws JAXBException, NspServiceException{
        JAXBContext jaxbContext = JAXBContext.newInstance(AuthorizationCodeRequestType.class, AuthorizationResponseType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class);

        // Create request
        AuthorizationCodeRequestType request = new ObjectFactory().createAuthorizationCodeRequestType();
        request.setAuthorizationCode(authorisationCode);

        JAXBElement<AuthorizationCodeRequestType> requestXml = new ObjectFactory().createAuthorizationCodeRequestStructure(request);
        AuthorizationResponseType authorisationResponse = null;
        try {
            authorisationResponse = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/sdm/Gateway/", new URI(saesEndpoint + "/service/AuthorizationCodeService"), AuthorizationResponseType.class);
        }catch (URISyntaxException e){
            log.debug("SaesEndpoint is not a valid URI");
        }
        return authorisationResponse;
    }

    public authorization20240105.AuthorizationResponseType callAuthorise20240105(String cpr, Identity identity, String saesEndpoint) throws JAXBException, NspServiceException{
        JAXBContext jaxbContext = JAXBContext.newInstance(authorization20240105.AuthorizationRequestType.class, authorization20240105.AuthorizationResponseType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class, authorization.Header.class, Security.class, new QName("http://nsi.dk/-/stamdata/3.0", "AuthorizationResponseStructure").getClass());

        // Create request
        authorization20240105.AuthorizationRequestType request = new authorization20240105.ObjectFactory().createAuthorizationRequestType();
        request.setCpr(cpr);

        JAXBElement<authorization20240105.AuthorizationRequestType> requestXml = new authorization20240105.ObjectFactory().createAuthorizationRequestStructure(request);
        authorization20240105.AuthorizationResponseType authorisationResponse = null;
        try {
            authorisationResponse = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/sdm/Gateway/", new URI(saesEndpoint + "/service/AuthorizationService-20240105"), authorization20240105.AuthorizationResponseType.class);
        }catch (URISyntaxException e){
            log.debug("SaesEndpoint is not a valid URI");
        }
        return authorisationResponse;
    }

    public authorization20240105.AuthorizationResponseType callAuthoriseCode20240105(String authorisationCode, Identity identity, String saesEndpoint) throws JAXBException, NspServiceException{
        JAXBContext jaxbContext = JAXBContext.newInstance(authorization20240105.AuthorizationCodeRequestType.class, authorization20240105.AuthorizationResponseType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class);

        // Create request
        authorization20240105.AuthorizationCodeRequestType request = new authorization20240105.ObjectFactory().createAuthorizationCodeRequestType();
        request.setAuthorizationCode(authorisationCode);

        JAXBElement<authorization20240105.AuthorizationCodeRequestType> requestXml = new authorization20240105.ObjectFactory().createAuthorizationCodeRequestStructure(request);
        authorization20240105.AuthorizationResponseType authorisationResponse = null;
        try {
            authorisationResponse = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/sdm/Gateway/", new URI(saesEndpoint + "/service/AuthorizationCodeService-20240105"), authorization20240105.AuthorizationResponseType.class);
        }catch (URISyntaxException e){
            log.debug("SaesEndpoint is not a valid URI");
        }
        return authorisationResponse;
    }

}
