/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.client;

import dk.nsi.stamdata.configuration.NspServiceException;
import dk.nsp.test.idp.model.Identity;
import filterManagement.*;
import org.apache.log4j.Logger;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

public class FilterManagementClient extends AbstractSDMClient {

    private Logger log = Logger.getLogger(getClass());

    public FilterResponseType callFilterManagementNew(String filterName, String filterViewPath, List<String> dataTypeFieldList, Identity identity, String sfmEndpoint) throws JAXBException, NspServiceException {
        JAXBContext jaxbContext = JAXBContext.newInstance(FilterResponseType.class, FilterNewRequestType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class, Security.class, new QName("http://nsi.dk/-/stamdata/3.0", "FilterResponseStructure").getClass());

        // Create request
        FilterNewRequestType request = new filterManagement.ObjectFactory().createFilterNewRequestType();
        request.setName(filterName);
        request.setViewPath(filterViewPath);
        request.getDataTypeFieldList().addAll(dataTypeFieldList);

        //Ready request of type: FilterNewRequestType
        QName stamdataFilterServiceQName = new QName("http://nsi.dk/2024/04/03/StamdataSfm", "FilterNewRequestType");
        JAXBElement<FilterNewRequestType> requestXml = new JAXBElement<>(stamdataFilterServiceQName, FilterNewRequestType.class, request);

        FilterResponseType response = null;
        try {
            response = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/2024/04/03/StamdataSfm", new URI(sfmEndpoint), FilterResponseType.class);
        }catch (URISyntaxException e){
            log.debug("SfmsEndpoint is not a valid URI");
        }
        return response;
    }

    public FilterResponseType callFilterManagementDelete(String filterName, Identity identity, String sfmEndpoint) throws JAXBException, NspServiceException{
        JAXBContext jaxbContext = JAXBContext.newInstance(FilterResponseType.class, FilterNewRequestType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class, Security.class, new QName("http://nsi.dk/-/stamdata/3.0", "FilterResponseStructure").getClass());

        // Create request
        FilterNewRequestType request = new filterManagement.ObjectFactory().createFilterNewRequestType();
        request.setName(filterName);

        //Ready request of type: FilterDeleteRequestType
        QName stamdataFilterServiceQName = new QName("http://nsi.dk/2024/04/03/StamdataSfm", "FilterDeleteRequestType");
        JAXBElement<FilterNewRequestType> requestXml = new JAXBElement<>(stamdataFilterServiceQName, FilterNewRequestType.class, request);

        FilterResponseType response = null;
        try {
            response = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/2024/04/03/StamdataSfm", new URI(sfmEndpoint), FilterResponseType.class);
        }catch (URISyntaxException e){
            log.debug("SfmsEndpoint is not a valid URI");
        }
        return response;
    }
}
