/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.client;

import FunctionalExt.Either;
import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.nsi.stamdata.configuration.NspServiceException;
import dk.nsp.test.idp.model.Identity;
import oio.medcom.cprservice._1_0_4.GetPersonInformationOut;
import oio.medcom.cprservice._1_0_4_1a.GetPersonInformationIn;
import oio.medcom.cprservice._1_0_4_1a.ObjectFactory;
import org.apache.log4j.Logger;
import org.joda.time.DateTime;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.soap.SOAPFault;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Date;

import static FunctionalExt.Uri.uri;
import static dk.nsi.stamdata.NspSoap.executeNspRequest;
import static dk.nsi.stamdata.NspSoap.parse;

public class LookupClient extends AbstractSDMClient {

    private Logger log = Logger.getLogger(getClass());

    public oio.medcom.cprservice._1_0_3.GetPersonInformationOut callGetPersonInformation103(String cpr, Identity identity, String scesEndpoint) throws JAXBException, NspServiceException {
        JAXBContext jaxbContext = JAXBContext.newInstance( oio.medcom.cprservice._1_0_3.GetPersonInformationOut.class,  oio.medcom.cprservice._1_0_3.GetPersonInformationIn.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class);

        // Create request
        oio.medcom.cprservice._1_0_3.GetPersonInformationIn request = new oio.medcom.cprservice._1_0_3.ObjectFactory().createGetPersonInformationIn();
        request.setPersonCivilRegistrationIdentifier(cpr);

        oio.medcom.cprservice._1_0_3.GetPersonInformationOut lookupResponse = null;
        try {
            lookupResponse = super.call(identity, request, jaxbContext, "urn:oio:medcom:cprservice:1.0.3#getPersonInformation", new URI(scesEndpoint));
        }catch (URISyntaxException e){
            log.debug("ScesEndpoint is not a valid URI");
        }
        return lookupResponse;
    }

    public oio.medcom.cprservice._1_0_4.GetPersonInformationOut callGetPersonInformation104(String cpr, Identity identity, String scesEndpoint) throws JAXBException, NspServiceException {
        JAXBContext jaxbContext = JAXBContext.newInstance(oio.medcom.cprservice._1_0_4.GetPersonInformationOut.class, oio.medcom.cprservice._1_0_4.GetPersonInformationIn.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class);

        // Create request
        oio.medcom.cprservice._1_0_4.GetPersonInformationIn request = new oio.medcom.cprservice._1_0_4.ObjectFactory().createGetPersonInformationIn();
        request.setPersonCivilRegistrationIdentifier(cpr);

        oio.medcom.cprservice._1_0_4.GetPersonInformationOut lookupResponse = null;
        try {
            lookupResponse = super.call(identity, request, jaxbContext, "urn:oio:medcom:cprservice:1.0.4#getPersonInformation", new URI(scesEndpoint));
        }catch (URISyntaxException e){
            log.debug("ScesEndpoint is not a valid URI");
        }
        return lookupResponse;
    }

    public static Either<SOAPFault, oio.medcom.cprservice._1_0_3.GetPersonInformationOut> callGetPersonInformation103(String cpr, Identity identity, URI host) {
        oio.medcom.cprservice._1_0_3.GetPersonInformationIn request = new oio.medcom.cprservice._1_0_3.ObjectFactory().createGetPersonInformationIn();
        request.setPersonCivilRegistrationIdentifier(cpr);
        return callGetPersonInformation(
                uri(host + "/service/DetGodeCPROpslag-1.0.3").ifLeft(ex -> { throw new RuntimeException(ex); }),
                "urn:oio:medcom:cprservice:1.0.3#getPersonInformation",
                oio.medcom.cprservice._1_0_3.GetPersonInformationOut.class,
                identity,
                request);
    }

    public static Either<SOAPFault, GetPersonInformationOut> callGetPersonInformation104(String cpr, Identity identity, URI host) {
        URI uri;
        try { uri = new URI(host + "/service/DetGodeCPROpslag-1.0.4"); }
        catch (URISyntaxException e) { throw new RuntimeException(e); }
        oio.medcom.cprservice._1_0_4.GetPersonInformationIn request = new oio.medcom.cprservice._1_0_4.ObjectFactory().createGetPersonInformationIn();
        request.setPersonCivilRegistrationIdentifier(cpr);
        return callGetPersonInformation(
                uri(host + "/service/DetGodeCPROpslag-1.0.4").ifLeft(ex -> { throw new RuntimeException(ex); }),
                "urn:oio:medcom:cprservice:1.0.4#getPersonInformation",
                GetPersonInformationOut.class,
                identity,
                request);
    }

    public static Either<SOAPFault, oio.medcom.cprservice._1_0_4a.GetPersonInformationOut> callGetPersonInformation104a(
            String cpr, Identity identity, URI host) {
        oio.medcom.cprservice._1_0_4a.GetPersonInformationIn request = new oio.medcom.cprservice._1_0_4a.ObjectFactory().createGetPersonInformationIn();
        request.setPersonCivilRegistrationIdentifier(cpr);
        return callGetPersonInformation(
                uri(host + "/service/DetGodeCPROpslag-1.0.4a").ifLeft(ex -> { throw new RuntimeException(ex); }),
                "urn:oio:medcom:cprservice:1.0.4a#getPersonInformation",
                oio.medcom.cprservice._1_0_4a.GetPersonInformationOut.class,
                identity,
                request);
    }

    public static Either<SOAPFault, oio.medcom.cprservice._1_0_4_1.GetPersonInformationOut> callGetPersonInformation1041(
            String cpr, Identity identity, URI host) {
        oio.medcom.cprservice._1_0_4_1.GetPersonInformationIn request =
                new oio.medcom.cprservice._1_0_4_1.ObjectFactory().createGetPersonInformationIn();
        request.setPersonCivilRegistrationIdentifier(cpr);
        return callGetPersonInformation(
                uri(host + "/service/DetGodeCPROpslag-1.0.4.1").ifLeft(ex -> { throw new RuntimeException(ex); }),
                "urn:oio:medcom:cprservice:1.0.4.1#getPersonInformation",
                oio.medcom.cprservice._1_0_4_1.GetPersonInformationOut.class,
                identity,
                request);
    }

    public static Either<SOAPFault, GetPersonInformationOut> callGetPersonInformation1041a(
            String cpr, Identity identity, URI host) {
        GetPersonInformationIn request =
                new ObjectFactory().createGetPersonInformationIn();
        request.setPersonCivilRegistrationIdentifier(cpr);
        return callGetPersonInformation(
                uri(host + "/service/DetGodeCPROpslag-1.0.4.1a").ifLeft(ex -> { throw new RuntimeException(ex); }),
                "urn:oio:medcom:cprservice:1.0.4.1a#getPersonInformation",
                GetPersonInformationOut.class,
                identity,
                request);
    }

    public static <T, R> Either<SOAPFault, R> callGetPersonInformation(URI uri,
                                                                       String action,
                                                                       Class<R> c,
                                                                       Identity identity,
                                                                       T req) {
        return executeNspRequest(uri, action, identity, req)
                .flatMap(res -> parse(res, c).mapLeft(Either::<RuntimeException, SOAPFault>left))
                .mapLeft(l -> l.ifLeft(rte -> { throw rte; }));
    }

    public oio.medcom.cprservice._1_0_4a.GetPersonInformationOut callGetPersonInformation104a(String cpr, Identity identity, String scesEndpoint) throws JAXBException, NspServiceException {
        JAXBContext jaxbContext = JAXBContext.newInstance(oio.medcom.cprservice._1_0_4a.GetPersonInformationOut.class, oio.medcom.cprservice._1_0_4a.GetPersonInformationIn.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class);

        // Create request
        oio.medcom.cprservice._1_0_4a.GetPersonInformationIn request = new oio.medcom.cprservice._1_0_4a.ObjectFactory().createGetPersonInformationIn();
        request.setPersonCivilRegistrationIdentifier(cpr);

        oio.medcom.cprservice._1_0_4a.GetPersonInformationOut lookupResponse = null;
        try {
            lookupResponse = super.call(identity, request, jaxbContext, "urn:oio:medcom:cprservice:1.0.4a#getPersonInformation", new URI(scesEndpoint));
        }catch (URISyntaxException e){
            log.debug("ScesEndpoint is not a valid URI");
        }
        return lookupResponse;
    }

    public oio.medcom.cprservice._1_0_4_1.GetPersonInformationOut callGetPersonInformation1041(String cpr, Identity identity, String scesEndpoint) throws JAXBException, NspServiceException {
        JAXBContext jaxbContext = JAXBContext.newInstance(oio.medcom.cprservice._1_0_4_1.GetPersonInformationOut.class, oio.medcom.cprservice._1_0_4_1.GetPersonInformationIn.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class);

        // Create request
        oio.medcom.cprservice._1_0_4_1.GetPersonInformationIn request = new oio.medcom.cprservice._1_0_4_1.ObjectFactory().createGetPersonInformationIn();
        request.setPersonCivilRegistrationIdentifier(cpr);

        oio.medcom.cprservice._1_0_4_1.GetPersonInformationOut lookupResponse = null;
        try {
            lookupResponse = super.call(identity, request, jaxbContext, "urn:oio:medcom:cprservice:1.0.4.1#getPersonInformation", new URI(scesEndpoint));
        }catch (URISyntaxException e){
            log.debug("ScesEndpoint is not a valid URI");
        }
        return lookupResponse;
    }

    public oio.medcom.cprservice._1_0_4_1a.GetPersonInformationOut callGetPersonInformation1041a(String cpr, Identity identity, String scesEndpoint) throws JAXBException, NspServiceException {
        JAXBContext jaxbContext = JAXBContext.newInstance(oio.medcom.cprservice._1_0_4_1a.GetPersonInformationOut.class, oio.medcom.cprservice._1_0_4_1a.GetPersonInformationIn.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class);

        // Create request
        oio.medcom.cprservice._1_0_4_1a.GetPersonInformationIn request = new oio.medcom.cprservice._1_0_4_1a.ObjectFactory().createGetPersonInformationIn();
        request.setPersonCivilRegistrationIdentifier(cpr);

        oio.medcom.cprservice._1_0_4_1a.GetPersonInformationOut lookupResponse = null;
        try {
            lookupResponse = super.call(identity, request, jaxbContext, "urn:oio:medcom:cprservice:1.0.4.1a#getPersonInformation", new URI(scesEndpoint));
        }catch (URISyntaxException e){
            log.debug("ScesEndpoint is not a valid URI");
        }
        return lookupResponse;
    }

    public dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType callGetPersonDetails101(String birthday, Identity identity, String scesEndpoint) throws JAXBException, NspServiceException {
        JAXBContext jaxbContext = JAXBContext.newInstance(dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType.class, dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class);

        // Create request
        dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType request = new dk.nsi._2011._09._23.stamdatacpr.ObjectFactory().createPersonLookupRequestType();
        request.setBirthDatePersonQuery(newXMLGregorianCalendar( new Date(birthday)));
        JAXBElement<dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType> requestXml = new dk.nsi._2011._09._23.stamdatacpr.ObjectFactory().createPersonLookupRequest(request);

        dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType lookupResponse = null;
        try {
            lookupResponse = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/2014/03/01/StamdataCpr/getPersonDetails", new URI(scesEndpoint), dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType.class);
        }catch (URISyntaxException e){
            log.debug("ScesEndpoint is not a valid URI");
        }
        return lookupResponse;
    }

    public dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType callGetPersonDetails110(String birthday, Identity identity, String scesEndpoint) throws JAXBException, NspServiceException {
        JAXBContext jaxbContext = JAXBContext.newInstance(dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType.class, dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class);

        // Create request
        dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType request = new dk.nsi._2011._09._23.stamdatacpr.ObjectFactory().createPersonLookupRequestType();
        request.setBirthDatePersonQuery(newXMLGregorianCalendar( new Date(birthday)));
        JAXBElement<dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType> requestXml = new dk.nsi._2011._09._23.stamdatacpr.ObjectFactory().createPersonLookupRequest(request);

        dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType lookupResponse = null;
        try {
            lookupResponse = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/2018/06/08/StamdataCpr/getPersonDetails", new URI(scesEndpoint), dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType.class);
        }catch (URISyntaxException e){
            log.debug("ScesEndpoint is not a valid URI");
        }
        return lookupResponse;
    }

    public dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType callGetPersonDetailsWithConsent110(dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType request, Identity identity, String scesEndpoint) throws JAXBException, NspServiceException {
        JAXBContext jaxbContext = JAXBContext.newInstance(dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType.class, dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class);

        JAXBElement<dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType> requestXml = new dk.nsi._2011._09._23.stamdatacpr.ObjectFactory().createPersonLookupRequest(request);

        dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType lookupResponse = null;
        try {
            lookupResponse = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/2018/06/08/StamdataCpr/getPersonDetailsWithConsent", new URI(scesEndpoint), dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType.class);
        }catch (URISyntaxException e){
            log.debug("ScesEndpoint is not a valid URI");
        }
        return lookupResponse;
    }

    public dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType callGetPersonDetails120(String birthday, Identity identity, String scesEndpoint) throws JAXBException, NspServiceException {
        JAXBContext jaxbContext = JAXBContext.newInstance(dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType.class, dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class);

        // Create request
        dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType request = new dk.nsi._2011._09._23.stamdatacpr.ObjectFactory().createPersonLookupRequestType();
        request.setBirthDatePersonQuery(newXMLGregorianCalendar( new Date(birthday)));
        JAXBElement<dk.nsi._2011._09._23.stamdatacpr.PersonLookupRequestType> requestXml = new dk.nsi._2011._09._23.stamdatacpr.ObjectFactory().createPersonLookupRequest(request);

        dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType lookupResponse = null;
        try {
            lookupResponse = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/2020/10/01/StamdataCpr/getPersonDetails", new URI(scesEndpoint), dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType.class);
        }catch (URISyntaxException e){
            log.debug("ScesEndpoint is not a valid URI");
        }
        return lookupResponse;
    }

    public static XMLGregorianCalendar newXMLGregorianCalendar(Date date)
    {
        DatatypeFactory factory;
        try {
            factory = DatatypeFactory.newInstance();
        } catch (DatatypeConfigurationException e) {
            throw new RuntimeException(e);
        }
        return factory.newXMLGregorianCalendar(new DateTime(date).toGregorianCalendar());
    }
}