/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.client;

import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.nsi.stamdata.configuration.NspServiceException;
import dk.nsp.test.idp.model.Identity;
import multilookup20240227.*;
import org.apache.log4j.Logger;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import java.net.URI;
import java.net.URISyntaxException;

public class MultiLookup20240227Client extends AbstractSDMClient {
    
    private Logger log = Logger.getLogger(getClass());

    public ReplicationResponseType callMultiLookup(String cpr, String register, String dataType,  String filterName, Identity identity, String srfsEndpoint) throws JAXBException, NspServiceException {
        JAXBContext jaxbContext = null;
        try {
            jaxbContext = JAXBContext.newInstance(ReplicationResponseType.class, ReplicationRequestType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class, multilookup20240227.Header.class, multilookup20240227.Security.class);
        } catch (JAXBException e) {
            throw new RuntimeException(e);
        }

        // Create request
        ReplicationRequestType request = getReplicationRequestType(cpr, filterName, register, dataType);
        JAXBElement<ReplicationRequestType> requestXml = new ObjectFactory().createReplicationRequest(request);

        ReplicationResponseType response = null;
        try {
            response = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/2024/02/27/StamdataRfs/replicate", new URI(srfsEndpoint + "-20240227"), ReplicationResponseType.class);
        }catch (URISyntaxException e){
            log.debug("ScesEndpoint is not a valid URI");
        }

        return response;
    }

    private static ReplicationRequestType getReplicationRequestType(String cpr, String filterName, String register, String dataType) {
        // Create request
        ReplicationRequestType request = new ObjectFactory().createReplicationRequestType();
        request.setRegister(register);
        request.setDatatype(dataType);
        request.setVersion(2L);
        request.setRegisterVersion(1L);
        IdList idList = new IdList();
        idList.getId().add(cpr);
        request.setIdList(idList);
        if (filterName != null && !filterName.isEmpty()) {
            request.setFilter(filterName);
        } else {
            request.setFilter(null);
        }
        return request;
    }
}
