/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.client;

import dk.nsi.stamdata.configuration.NspServiceException;
import dk.nsp.test.idp.model.Identity;
import org.apache.log4j.Logger;
import replication.ReplicationRequestType;
import replication.ReplicationResponseType;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import java.net.URI;
import java.net.URISyntaxException;

public class ReplicationClient extends AbstractSDMClient {

    private Logger log = Logger.getLogger(getClass());

    public ReplicationResponseType callStamdataReplication( Identity identity, String skrsEndpoint) throws JAXBException, NspServiceException{
        JAXBContext jaxbContext = JAXBContext.newInstance(ReplicationResponseType.class, ReplicationRequestType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, replication.Header.class, replication.Security.class, replication.AssertionType.class, replication.NameIDType.class);

        // Create request
        ReplicationRequestType request = new replication.ObjectFactory().createReplicationRequestType();
        request.setVersion(2L);
        request.setDatatype("person");
        request.setRegister("cpr");
        request.setRegisterVersion(1L);
        request.setOffset("");
        JAXBElement<ReplicationRequestType> requestXml = new replication.ObjectFactory().createReplicationRequest(request);

        ReplicationResponseType replicationResponse = null;
        try {
            replicationResponse = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/2011/10/21/StamdataKrs/replicate", new URI(skrsEndpoint), ReplicationResponseType.class);
        }catch (URISyntaxException e){
            log.debug("SkrsEndpoint is not a valid URI");
        }
        return replicationResponse;
    }


}
