/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.client;

import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.nsi.stamdata.configuration.NspServiceException;
import dk.nsp.test.idp.model.Identity;
import org.apache.log4j.Logger;
import yder.YderRequestType;
import yder.YderResponseType;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import java.net.URI;
import java.net.URISyntaxException;

public class YderLookupClient extends AbstractSDMClient {

    private Logger log = Logger.getLogger(getClass());

    public YderResponseType callYderLookup(String cpr, Identity identity, String syesEndpoint) throws JAXBException, NspServiceException{
        JAXBContext jaxbContext = JAXBContext.newInstance(YderResponseType.class, YderRequestType.class, org.xmlsoap.schemas.soap.envelope.Envelope.class, Header.class, yder.Header.class, yder.Security.class);

        // Create request
        YderRequestType request = new yder.ObjectFactory().createYderRequestType();
        request.setYdernummer(cpr);

        JAXBElement<YderRequestType> requestXml = new yder.ObjectFactory().createYderRequestStructure(request);
        YderResponseType lookupResponse = null;
        try {
            lookupResponse = super.callNoXmlRootElement(identity, requestXml, jaxbContext, "http://nsi.dk/sdm/Gateway/", new URI(syesEndpoint), YderResponseType.class);
        }catch (URISyntaxException e){
            log.debug("SyesEndpoint is not a valid URI");
        }
        return lookupResponse;
    }
}