/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.lookup;

import dk.nsi.stamdata.client.LookupClient;
import dk.nsi.stamdata.configuration.NspServiceException;
import dk.nsi.stamdata.configuration.SDMContextConfiguration;
import dk.nsp.test.idp.EmployeeIdentities;
import dk.nsp.test.idp.model.Identity;
import oio.medcom.cprservice._1_0_4.GetPersonInformationOut;
import org.junit.*;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.rules.SpringClassRule;
import org.springframework.test.context.junit4.rules.SpringMethodRule;

import javax.xml.bind.JAXBException;
import javax.xml.soap.SOAPFault;
import java.net.URI;
import java.net.URISyntaxException;

import static dk.nsi.stamdata.lookup.Helper.*;
import static org.junit.Assert.*;

@ContextConfiguration(classes = SDMContextConfiguration.class)
public class DenGodeCPROpslag104Test {


    @ClassRule
    public static final SpringClassRule scr = new SpringClassRule();

    @Rule
    public final SpringMethodRule smr = new SpringMethodRule();

    @Value("${sces.endpoint}")
    private String scesEndpoint;

    private LookupClient lookupClient;

    @Before
    public void setup() {
        lookupClient = new LookupClient();
    }

    @Test
    public void getPersonWithHealthCareInformation_Sucess() throws JAXBException, NspServiceException {

        // Given
        //create request
        String cpr = "0304784567";
        Identity identity = EmployeeIdentities.tandlægeAlanTuring();

        // When
        //cll service
        GetPersonInformationOut reponse = lookupClient.callGetPersonInformation104(cpr, identity,
                scesEndpoint + "/service/DetGodeCPROpslag-1.0.4");

        // Then
        //assert reponse okay?
        assertNotNull(reponse.getPersonInformationStructure());
        Assert.assertEquals("0304784567", reponse.getPersonInformationStructure().getRegularCPRPerson().getSimpleCPRPerson().getPersonCivilRegistrationIdentifier());
        Assert.assertEquals("0320", reponse.getPersonInformationStructure().getPersonAddressStructure().getAddressComplete().getAddressAccess().getMunicipalityCode());
    }

    @Test
    public void givenGetPersonInformationRequestWithNotWhitelistedCvrThenErrorShouldBeReturned() throws URISyntaxException {
        String someCpr = "0101800101", notWhitelistedCvr = "12345678";
        SOAPFault f = assertLeft(LookupClient
                .callGetPersonInformation104(someCpr, employeeIdentity(notWhitelistedCvr),
                        new URI(scesEndpoint)));
        assertGetPersonInformationWhitelistFault(f, notWhitelistedCvr);
    }
}