/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.lookup;

import FunctionalExt.Either;
import dk.nsp.test.idp.builder.Builders;
import dk.nsp.test.idp.model.EmployeeIdentity;
import dk.nsp.test.idp.vault.RemoteCredentialVault;
import dk.sosi.seal.model.constants.FaultCodeValues;

import javax.xml.soap.SOAPFault;
import java.net.URI;
import java.util.UUID;

import static org.junit.Assert.assertEquals;

public class Helper {
    public static void assertGetPersonInformationWhitelistFault(SOAPFault f, String expectedNotWhitelistedCvr) {
        assertEquals(
                "Dette CVR-nummer (" + expectedNotWhitelistedCvr + ") " +
                        "er ikke whitelistet til at hente de ønskede oplysniger",
                f.getFaultString());
        assertEquals("S:Server", f.getFaultCode());
        assertEquals(FaultCodeValues.NOT_AUTHORIZED, f.getDetail().getFirstChild().getFirstChild().getNodeValue());
    }

    public static EmployeeIdentity employeeIdentity(String cvr) {
        return Builders.employeeIdentity()
                .representing(Builders.employee()
                        .identifiedBy(UUID.fromString("4a4c9843-58b5-4694-95d0-ea5ab80823da")))
                .employedAt(Builders.organization().named("Sundhedsdatastyrelsen").identifiedBy(cvr))
                .usedBy(Builders.serviceConsumer().named("Service Consumer Test")
                        .identifiedBy(new RemoteCredentialVault(URI.create(
                                "https://www.nspop.dk/download/attachments/190481050" +
                                        "/NSP_Test_Service_Consumer_sds.p12"),
                                "Test1234".toCharArray(), true)))
                .verifiedBy(Builders.identityProvider()
                        .named("https://idp.test.nspop.dk")
                        .identifiedBy(new RemoteCredentialVault(URI.create(
                                "https://www.nspop.dk/download/attachments/190481050" +
                                        "/NSP_Test_Identity_Provider_sds.p12"),
                                "Test1234".toCharArray(), true)))
                .using(Builders.securityTokenService()
                        .at(URI.create("http://test1.ekstern-test.nspop.dk:8080/sts/services/BST2SOSI")))
                .build();
    }

    static <L, R> L assertLeft(Either<L, R> e) {
        return e.ifRight(
                r -> { throw new AssertionError("Either should be Left, but was Right(" + r.toString() + ')' ); });
    }
}