/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.lookup;

import dk.nsi._2014._03._01.stamdatacpr.PersonLookupResponseType;
import dk.nsi.stamdata.client.LookupClient;
import dk.nsi.stamdata.configuration.NspServiceException;
import dk.nsi.stamdata.configuration.SDMContextConfiguration;
import dk.nsp.test.idp.OrganizationIdentities;
import dk.nsp.test.idp.model.Identity;
import oio.medcom.cprservice._1_0.PersonInformationStructureType;
import org.junit.Before;
import org.junit.ClassRule;
import org.junit.Rule;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.rules.SpringClassRule;
import org.springframework.test.context.junit4.rules.SpringMethodRule;

import javax.xml.bind.JAXBException;

import java.util.List;
import static org.junit.Assert.assertTrue;

@ContextConfiguration(classes = SDMContextConfiguration.class )
public class StamdataPersonLookup101Test {



    @ClassRule
    public static final SpringClassRule scr = new SpringClassRule();

    @Rule
    public final SpringMethodRule smr = new SpringMethodRule();

    @Value("${sces.endpoint}")
    private String scesEndpoint;

    private LookupClient lookupClient;
    Identity identity = OrganizationIdentities.sundhedsdatastyrelsen();

    @Before
    public void setup() {
        lookupClient = new LookupClient();
    }

    @Test
    public void getPersonDetails_Sucess() throws JAXBException, NspServiceException {

        // Given
        String expectedName = "Egon";
        //create request
        String birthdate = "04/03/1978"; // month/day/year
        //in doc birthdayte instead of cpr 

        // When
        PersonLookupResponseType reponse = lookupClient.callGetPersonDetails101(birthdate, identity, scesEndpoint + "/service/StamdataPersonLookup-1.0.1");

        // Then
        assertTrue(reponse.getPersonInformationStructure().size()>0);
        assertTrue(findName(reponse.getPersonInformationStructure(), expectedName));
    }

    private boolean findName(List<PersonInformationStructureType> personList, String expectedGivenName) {
        for (PersonInformationStructureType elm : personList) {
            if (elm.getRegularCPRPerson().getSimpleCPRPerson().getPersonNameStructure().getPersonGivenName().equals(expectedGivenName)) {
                return true;
            }
        }
        return false;
    }
    
}
