/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.multiLookup;

import dk.nsi.stamdata.client.MultiLookupClient;
import dk.nsi.stamdata.configuration.NspServiceException;
import dk.nsi.stamdata.configuration.SDMContextConfiguration;
import dk.nsp.test.idp.OrganizationIdentities;
import dk.nsp.test.idp.model.Identity;
import dk.sosi.seal.model.AuthenticationLevel;
import multilookup.ReplicationResponseType;
import org.junit.*;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.rules.SpringClassRule;
import org.springframework.test.context.junit4.rules.SpringMethodRule;

import javax.xml.bind.JAXBException;
import javax.xml.xpath.XPathExpressionException;

@RunWith(Parameterized.class)
@ContextConfiguration(classes = SDMContextConfiguration.class )
public class MultiLookupIntegrationTest {

    @ClassRule
    public static final SpringClassRule scr = new SpringClassRule();

    @Rule
    public final SpringMethodRule smr = new SpringMethodRule();

    @Parameterized.Parameter(0)
    public AuthenticationLevel authenticationLevel;

    @Value("${srfs.endpoint}")
    private String srfsEndpoint;

    @Value("${sfm.endpoint}")
    private String sfmEndpoint;

    private MultiLookupClient multiLookupClient;
    Identity identity = OrganizationIdentities.sundhedsdatastyrelsen();
    private MultiLookupTestHelper helper;

    @Parameterized.Parameters(name = "{0}")
    public static AuthenticationLevel[] data() {
        return new AuthenticationLevel[]{AuthenticationLevel.VOCES_TRUSTED_SYSTEM};
    }

    @Before
    public void setup() {
        multiLookupClient = new MultiLookupClient();
        helper = new MultiLookupTestHelper();
    }

    @Test
    public void getPersonWithAuthorization() throws JAXBException, NspServiceException, XPathExpressionException {

        // Given
        String cpr = "0304784567";

        // When
        ReplicationResponseType response = multiLookupClient.callMultiLookup(cpr, identity, srfsEndpoint);

        // Then
        Assert.assertNotNull(response.getAny());
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.CPR, cpr);
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.KOEN, "M");
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.FORNAVN, "Egon");
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.EFTERNAVN, "Olsen");
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.BYNAVN, "Aarhus");
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.VEJNAVN, "Bogenhøjvej");
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.KOMMUNEKODE, "320");
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.VEJKODE, "6503");
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.FOEDSELSDATO, "1978-04-03");
    }

    @Test
    public void getPersonWithHistoricData() throws JAXBException, NspServiceException, XPathExpressionException {
        Assume.assumeTrue(sfmEndpoint.contains("localhost"));

        // Given
        String cpr = "0304784569";

        // When
        ReplicationResponseType response = multiLookupClient.callMultiLookup(cpr, identity, srfsEndpoint);

        // Then
        Assert.assertNotNull(response.getAny());
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.CPR, cpr);
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.FORNAVN, "Henning");
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.EFTERNAVN, "Primdahl");
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.VEJNAVN, "Nyvej");
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.VALIDFROM, "2024-07-10T23:59:59.000+02:00");
        helper.assertResponseContainsValue(response, "person", MultiLookupTestHelper.ReplicationResponseNodes.VALIDTO, "9999-12-31T23:59:59.000+01:00");
    }
}