/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsi.stamdata.multiLookup;

import dk.nsi.stamdata.testing.NamespaceContextMap;
import org.w3c.dom.Element;

import javax.xml.namespace.NamespaceContext;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import static org.junit.Assert.assertEquals;

public class MultiLookupTestHelper {
    public String getViewPath(String register, long version, String dataType, long registerverion) {
        // viewPath format: register/version/datatype/Registerversion
        // eks with Registerversion:    vitamin/1/grunddata/1
        // eks without Registerversion: vitamin/1/grunddata/
        if (registerverion == 0L) {
            return String.format("%s/%d/%s/", register, version, dataType);
        } else {
            return String.format("%s/%d/%s/%d", register, version, dataType, registerverion);
        }
    }

    public enum ReplicationResponseNodes {
        CPR("cpr"),
        KOEN("koen"),
        FORNAVN("fornavn"),
        EFTERNAVN("efternavn"),
        VEJNAVN("vejnavn"),
        BYGNINGSNUMMER("bygningsnummer"),
        BYNAVN("bynavn"),
        POSTNUMMER("postnummer"),
        POSTDISTRIKT("postdistrikt"),
        STATUS("status"),
        GAELDENDECPR("gaeldendeCPR"),
        FOEDSELSDATO("foedselsdato"),
        STILLING("stilling"),
        VEJKODE("vejKode"),
        KOMMUNEKODE("kommuneKode"),
        VALIDFROM("validFrom"),
        VALIDTO("validTo");

        private final String nodeName;

        ReplicationResponseNodes(String nodeName) {
            this.nodeName = nodeName;
        }

        public String getNodeName() {
            return this.nodeName;
        }
    }

    public void assertResponseContainsValue(multilookup20240227.ReplicationResponseType response, String dataType, ReplicationResponseNodes node, String expectedValue) throws XPathExpressionException {
        Element anyAsElement = (Element) response.getAny();
        assertResponseContainsValue(anyAsElement, dataType, node, expectedValue, "2024/02/27");
    }

    public void assertResponseContainsValue(multilookup.ReplicationResponseType response, String dataType, ReplicationResponseNodes node, String expectedValue) throws XPathExpressionException {
        Element anyAsElement = (Element) response.getAny();
        assertResponseContainsValue(anyAsElement, dataType, node, expectedValue, "2011/10/21");
    }

    private void assertResponseContainsValue(Element anyAsElement, String dataType, ReplicationResponseNodes node, String expectedValue, String rfsVersion) throws XPathExpressionException {
        String xPath = String.format("//%s:%s/%s:%s", dataType, dataType, dataType, node.getNodeName());
        XPathExpression expression = createXpathExpression(xPath, rfsVersion);
        String result = expression.evaluate(anyAsElement);
        assertEquals(expectedValue, result);
    }

    private XPathExpression createXpathExpression(String expression, String rfsVersion) throws XPathExpressionException {
        NamespaceContext context = new NamespaceContextMap(
                "rfs", "http://nsi.dk/" + rfsVersion + "/StamdataRfs/",
                "atom", "http://www.w3.org/2005/Atom",
                "sdm", "http://nsi.dk/-/stamdata/3.0/cpr",
                "sikrede", "http://nsi.dk/-/stamdata/3.0/sikrede",
                "yder", "http://nsi.dk/-/stamdata/3.0/yderregister",
                "grunddata", "http://nsi.dk/-/stamdata/3.0/vitamin",
                "forhoejettakst", "http://nsi.dk/-/stamdata/3.0/tilskudsblanket",
                "bemyndigelse", "http://nsi.dk/-/stamdata/3.0/bemyndigelsesservice",
                "dosageproposal", "http://nsi.dk/-/stamdata/3.0/doseringsforslag",
                "doseringsforslag", "http://nsi.dk/-/stamdata/3.0/doseringsforslag",
                "person", "http://nsi.dk/-/stamdata/3.0/cpr");
        XPathFactory factory = XPathFactory.newInstance();
        XPath xpath = factory.newXPath();
        xpath.setNamespaceContext(context);
        return xpath.compile(expression);
    }
}
