/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.personalinformation;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.junit.Assert;
import org.openapitools.model.Age;
import org.openapitools.model.Custody;
import org.openapitools.model.Person;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;


public class IntegrationTestConfiguration {
    private final String personinformationEndpoint;

    public IntegrationTestConfiguration(String personinformationEndpoint) {
        this.personinformationEndpoint = personinformationEndpoint;
    }

    private <T> T callService(T value, String format, String version, String cpr) {
        try (DefaultHttpClient client = new DefaultHttpClient()){
            String extension = String.format(format, version, cpr);
            HttpGet request = new HttpGet(personinformationEndpoint+extension);
            HttpResponse response = client.execute(request);

            String jsonString = EntityUtils.toString(response.getEntity());
            //We will deserialize the Response body into Token Response
            ObjectMapper mapper = new ObjectMapper();
            value = mapper.readValue(jsonString, (Class<T>)value.getClass());
        } catch (IOException e) {
            Assert.fail("Should not cast exception: " + e.getMessage());
        }
        return value;
    }

    public Person callPersonInformationService(String version, String cpr)  {
       return callService(new Person(),"/%s/person/%s", version, cpr);
    }

    public <T> T callPersonInformationStatusService(T valueType, String version, String cpr)  {
        return callService(valueType,"/%s/person/%s/status", version, cpr);
    }

    public Age callPersonInformationAgeService(String version, String cpr)  {
        return callService(new Age(),"/%s/person/%s/age", version, cpr);
    }

    public  Map<String,String> callPersonInformationBirthdayService(String version, String cpr)  {
        return callService(new HashMap<>(),"/%s/person/%s/birthday", version, cpr);
    }

    public Custody callPersonInformationCustodyService(String version, String cpr)  {
        return callService(new Custody(),"/%s/person/%s/custody", version, cpr);
    }

    public Map<String,String> callPersonInformationNameService(String version, String cpr)  {
        return callService(new HashMap<>(),"/%s/person/%s/name", version, cpr);
    }

    public Map<String,String> callPersonInformationAddressService(String version, String cpr)  {
        return callService(new HashMap<>(),"/%s/person/%s/address", version, cpr);
    }
}
