/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.yderLookup;

import dk.nsi.stamdata.client.YderLookupClient;
import dk.nsi.stamdata.configuration.NspServiceException;
import dk.nsi.stamdata.configuration.SDMContextConfiguration;
import dk.nsp.test.idp.OrganizationIdentities;
import dk.nsp.test.idp.model.Identity;
import dk.sosi.seal.model.AuthenticationLevel;
import org.junit.*;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.rules.SpringClassRule;
import org.springframework.test.context.junit4.rules.SpringMethodRule;
import yder.YderResponseType;
import javax.xml.bind.JAXBException;

@RunWith(Parameterized.class)
@ContextConfiguration(classes = SDMContextConfiguration.class )
public class YderServiceTest {



    @ClassRule
    public static final SpringClassRule scr = new SpringClassRule();

    @Rule
    public final SpringMethodRule smr = new SpringMethodRule();

    @Parameterized.Parameter(0)
    public AuthenticationLevel authenticationLevel;


    @Value("${syes.endpoint}")
    private String syesEndpoint;

    private YderLookupClient yderLookupClient;
    Identity identity = OrganizationIdentities.sundhedsdatastyrelsen();

    @Parameterized.Parameters(name = "{0}")
    public static AuthenticationLevel[] data() {
        return new AuthenticationLevel[]{AuthenticationLevel.VOCES_TRUSTED_SYSTEM};
    }

    @Before
    public void setup() {
        yderLookupClient = new YderLookupClient();
    }

    @Test
    public void getOuter_Sucess() throws JAXBException, NspServiceException {

        // Given
        //create request
        String cpr = "44733";

        // When
        //cll service
        YderResponseType reponse = yderLookupClient.callYderLookup(cpr, identity, syesEndpoint);

        // Then
        Assert.assertEquals("6100", reponse.getYder().getPostnummer());
        Assert.assertEquals("Haderslev", reponse.getYder().getPostdistrikt());
        Assert.assertEquals("83",reponse.getYder().getAmtKode());
        Assert.assertEquals("Region Syddanmark",reponse.getYder().getAmtTekst());
        Assert.assertEquals("Lægehuset Nørregade 40",reponse.getYder().getPraksisbetegnelse());
        Assert.assertEquals("Nørregade 40",reponse.getYder().getAdresse());
    }
}
