/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.personinformation;

import com.google.inject.AbstractModule;
import com.google.inject.Module;
import com.google.inject.servlet.ServletModule;
import com.trifork.stamdata.ComponentMonitor;
import com.trifork.stamdata.ConfigurationLoader;
import com.trifork.stamdata.MonitoringModule;
import com.trifork.stamdata.persistence.PersistenceModule;
import org.apache.log4j.Logger;
import org.jboss.resteasy.plugins.guice.GuiceResteasyBootstrapServletContextListener;

import javax.servlet.ServletContext;
import java.util.Arrays;
import java.util.List;

import static com.google.inject.name.Names.bindProperties;

public class ComponentController extends GuiceResteasyBootstrapServletContextListener {
	public static final String COMPONENT_NAME = "stamdata-personinformation";
    private static Logger logger;

	@Override
	protected List<? extends Module> getModules(ServletContext context) {
		getLogger().info("Initializing modules");
		return Arrays.asList(new ComponentModule(), new ServiceModule());
	}

	public static class ComponentModule extends AbstractModule {
		@Override
		protected void configure() {
			// Load the components configuration and bind it to named
			// dependencies.
			bindProperties(binder(), ConfigurationLoader.loadForName(COMPONENT_NAME));
            getLogger().info("Configuring ComponentModule for " + COMPONENT_NAME);

            getLogger().info("Installing PersistenceModule");
			install(new PersistenceModule());
			install(new PersonInformationModule());
			install(new OpenApiModule());
		}
	}

	private static class ServiceModule extends ServletModule {
		@Override
		protected void configureServlets() {
            getLogger().info("Configuring ServiceModule for " + COMPONENT_NAME);

			// Tell the monitoring module how to monitor the component.
			// All monitor pages are bound to the URL /status.

			bind(ComponentMonitor.class).to(ComponentMonitorImpl.class);
            getLogger().info("Installing MonitoringModule");
			install(new MonitoringModule());
		}
	}

    private static Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger(ComponentController.class);
        }
        return logger;
    }
}
