/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.personinformation;

import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;

import org.codehaus.jackson.JsonGenerator;
import org.codehaus.jackson.JsonParser;
import org.codehaus.jackson.JsonProcessingException;
import org.codehaus.jackson.Version;
import org.codehaus.jackson.map.DeserializationContext;
import org.codehaus.jackson.map.JsonDeserializer;
import org.codehaus.jackson.map.JsonSerializer;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.map.SerializerProvider;
import org.codehaus.jackson.map.module.SimpleModule;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;

public class ExtendedPersonInformationMapper extends ObjectMapper {
// Denne anvendes, når standard mapning ikke er nok
	

// mapning af LocalDate/LocalDateTime fungerer i den gamle nuværende mapper. Alternativet med anvendelse af JavaTimeModule (JSR310Module) er forsøgt men lykkedes ikke pga. modstridende dependencies/versioner)
       public ExtendedPersonInformationMapper() {
           SimpleModule sm = new SimpleModule("PersonInformationModule", new Version(1,0,0,""));
           sm.addSerializer(LocalDate.class, new JsonSerializer<LocalDate>() {
               @Override
               public void serialize(LocalDate value, JsonGenerator jgen, SerializerProvider provider) throws IOException, JsonProcessingException {
                   ToStringSerializer.instance.serialize(value, jgen, provider);
               }
           });
           sm.addSerializer(LocalDateTime.class, new JsonSerializer<LocalDateTime>() {
               @Override
               public void serialize(LocalDateTime value, JsonGenerator jgen, SerializerProvider provider) throws IOException, JsonProcessingException {
                   ToStringSerializer.instance.serialize(value, jgen, provider);
               }
           });
           sm.addDeserializer(LocalDate.class, new JsonDeserializer<LocalDate>() {
               @Override
               public LocalDate deserialize(JsonParser jp, DeserializationContext ctxt) throws IOException, JsonProcessingException {
                   String text = jp.getText();
                   LocalDate date = LocalDate.parse(text);
                   return date;
               }
           });
           sm.addDeserializer(LocalDateTime.class, new JsonDeserializer<LocalDateTime>() {
               @Override
               public LocalDateTime deserialize(JsonParser jp, DeserializationContext ctxt) throws IOException, JsonProcessingException {
                   String text = jp.getText();
                   LocalDateTime dateTime = LocalDateTime.parse(text);
                   return dateTime;
               }
           });
           registerModule(sm);
       }
   
}
