/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.personinformation.service._2024._08._01;

import com.google.inject.Inject;
import com.trifork.stamdata.models.PersonVO;
import dk.nsi.stamdata.cpr.dao.CprDAO;
import dk.nsi.stamdata.personinformation.service.PersonApiServiceUtil;
import org.apache.log4j.Logger;
import org.openapitools.model.StatusCode;

import javax.ws.rs.PathParam;
import javax.ws.rs.core.Response;
import java.sql.SQLException;
import java.util.Date;

public class PersonApiServiceImpl extends dk.nsi.stamdata.personinformation.service.v1.PersonApiServiceImpl implements PersonApiService {
    private static final Logger logger = Logger.getLogger(PersonApiServiceImpl.class);

    @Inject
    public PersonApiServiceImpl(CprDAO cprDAO) {
        super(cprDAO);
    }

    @Override
    public Response getPersonStatus(@PathParam("cpr") String cpr) throws SQLException {
        PersonVO personVO = cprDAO.findCpr(cpr, new Date());
        if(personVO != null) {
            String personStatus = personVO.getStatus();
            if (personStatus != null) {
                if(PersonApiServiceUtil.statusToInteger(personStatus) >= 0){
                    StatusCode statusCode = new StatusCode();
                    statusCode.setStatusCode(personStatus);
                    return Response
                            .ok()
                            .entity(statusCode)
                            .build();
                }
            }
        }
        return Response
                .status(Response.Status.NOT_FOUND)
                .entity("{}")
                .header("Content-Type", "application/json")
                .build();
    }
}
