/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.stamdata.personinformation;

import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.Month;

import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.openapitools.model.Birthday;

public class ExtendedPersonInformationMapperTest {


	private ExtendedPersonInformationMapper subject;
	
	
	@Before
	public void setup() {
		subject = new ExtendedPersonInformationMapper();
	}
	
    @Test
    public void testDeserializeLocalDate() throws JsonParseException, JsonMappingException, IOException {
    	
    	// Given
        String json = "\"2022-09-16\"";
        LocalDate localDate = LocalDate.of(2022, Month.SEPTEMBER, 16);

        // When
        LocalDate result = subject.readValue(json, LocalDate.class);
        
        
        // Then
        Assert.assertEquals(localDate, result);
    }
    
    @Test
    public void testDeserializeLocalDateTime() throws JsonParseException, JsonMappingException, IOException {
    	
    	// Given
    	String json = "\"2022-09-16T10:34:02\"";
    	LocalDateTime localDateTime = LocalDateTime.of(2022, Month.SEPTEMBER, 16, 10, 34, 02);

        // When
    	LocalDateTime result = subject.readValue(json, LocalDateTime.class);
        
        
        // Then
        Assert.assertEquals(localDateTime, result);
    }
    
    @Test
    public void testDeserializeBirthday() throws JsonParseException, JsonMappingException, IOException {
    	
    	// Given
        String json = "{\"birthday\":\"2022-09-16\"}";
        LocalDate localDate = LocalDate.of(2022, Month.SEPTEMBER, 16);

        // When
        Birthday result = subject.readValue(json, Birthday.class);
        
        
        // Then
        Assert.assertEquals(localDate, result.getBirthday());
    }
	
    
    @Test
    public void testSerializeLocalDate() throws JsonParseException, JsonMappingException, IOException {
    	
    	// Given
        String json = "\"2022-09-16\"";
        LocalDate localDate = LocalDate.of(2022, Month.SEPTEMBER, 16);

        // When
        String result = subject.writeValueAsString(localDate);
        
        
        // Then
        Assert.assertEquals(json, result);
    }
    
    @Test
    public void testSerializeLocalDateTime() throws JsonParseException, JsonMappingException, IOException {
    	
    	// Given
        String json = "\"2022-09-16T10:34:02\"";
        LocalDateTime localDateTime = LocalDateTime.of(2022, Month.SEPTEMBER, 16, 10, 34, 02); 

        // When
        String result = subject.writeValueAsString(localDateTime);
        
        
        // Then
        Assert.assertEquals(json, result);
    }
    
    @Test
    public void testSerializeBirthday() throws JsonParseException, JsonMappingException, IOException {
    	
    	// Given
        String json = "{\"birthday\":\"2022-09-16\"}";
        LocalDate localDate = LocalDate.of(2022, Month.SEPTEMBER, 16);
        Birthday birthday = new Birthday();
        birthday.setBirthday(localDate);

        // When
        String result = subject.writeValueAsString(birthday);
        
        
        // Then
        Assert.assertEquals(json, result);
    }


}
